// Code generated by protoc-gen-go. DO NOT EDIT.
// source: runtime/internal/examplepb/example.proto

package examplepb

import (
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	empty "github.com/golang/protobuf/ptypes/empty"
	timestamp "github.com/golang/protobuf/ptypes/timestamp"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

// NumericEnum is one or zero.
type NumericEnum int32

const (
	// ZERO means 0
	NumericEnum_ZERO NumericEnum = 0
	// ONE means 1
	NumericEnum_ONE NumericEnum = 1
)

var NumericEnum_name = map[int32]string{
	0: "ZERO",
	1: "ONE",
}

var NumericEnum_value = map[string]int32{
	"ZERO": 0,
	"ONE":  1,
}

func (x NumericEnum) String() string {
	return proto.EnumName(NumericEnum_name, int32(x))
}

func (NumericEnum) EnumDescriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{0}
}

// DeepEnum is one or zero.
type ABitOfEverything_Nested_DeepEnum int32

const (
	// FALSE is false.
	ABitOfEverything_Nested_FALSE ABitOfEverything_Nested_DeepEnum = 0
	// TRUE is true.
	ABitOfEverything_Nested_TRUE ABitOfEverything_Nested_DeepEnum = 1
)

var ABitOfEverything_Nested_DeepEnum_name = map[int32]string{
	0: "FALSE",
	1: "TRUE",
}

var ABitOfEverything_Nested_DeepEnum_value = map[string]int32{
	"FALSE": 0,
	"TRUE":  1,
}

func (x ABitOfEverything_Nested_DeepEnum) String() string {
	return proto.EnumName(ABitOfEverything_Nested_DeepEnum_name, int32(x))
}

func (ABitOfEverything_Nested_DeepEnum) EnumDescriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{1, 0, 0}
}

type RepeatedResponseBodyOut_Response_ResponseType int32

const (
	// UNKNOWN
	RepeatedResponseBodyOut_Response_UNKNOWN RepeatedResponseBodyOut_Response_ResponseType = 0
	// A is 1
	RepeatedResponseBodyOut_Response_A RepeatedResponseBodyOut_Response_ResponseType = 1
	// B is 2
	RepeatedResponseBodyOut_Response_B RepeatedResponseBodyOut_Response_ResponseType = 2
)

var RepeatedResponseBodyOut_Response_ResponseType_name = map[int32]string{
	0: "UNKNOWN",
	1: "A",
	2: "B",
}

var RepeatedResponseBodyOut_Response_ResponseType_value = map[string]int32{
	"UNKNOWN": 0,
	"A":       1,
	"B":       2,
}

func (x RepeatedResponseBodyOut_Response_ResponseType) String() string {
	return proto.EnumName(RepeatedResponseBodyOut_Response_ResponseType_name, int32(x))
}

func (RepeatedResponseBodyOut_Response_ResponseType) EnumDescriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{3, 0, 0}
}

// SimpleMessage represents a simple message sent to the Echo service.
type SimpleMessage struct {
	Id                   string   `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *SimpleMessage) Reset()         { *m = SimpleMessage{} }
func (m *SimpleMessage) String() string { return proto.CompactTextString(m) }
func (*SimpleMessage) ProtoMessage()    {}
func (*SimpleMessage) Descriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{0}
}

func (m *SimpleMessage) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_SimpleMessage.Unmarshal(m, b)
}
func (m *SimpleMessage) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_SimpleMessage.Marshal(b, m, deterministic)
}
func (m *SimpleMessage) XXX_Merge(src proto.Message) {
	xxx_messageInfo_SimpleMessage.Merge(m, src)
}
func (m *SimpleMessage) XXX_Size() int {
	return xxx_messageInfo_SimpleMessage.Size(m)
}
func (m *SimpleMessage) XXX_DiscardUnknown() {
	xxx_messageInfo_SimpleMessage.DiscardUnknown(m)
}

var xxx_messageInfo_SimpleMessage proto.InternalMessageInfo

func (m *SimpleMessage) GetId() string {
	if m != nil {
		return m.Id
	}
	return ""
}

type ABitOfEverything struct {
	SingleNested        *ABitOfEverything_Nested   `protobuf:"bytes,25,opt,name=single_nested,json=singleNested,proto3" json:"single_nested,omitempty"`
	Uuid                string                     `protobuf:"bytes,1,opt,name=uuid,proto3" json:"uuid,omitempty"`
	Nested              []*ABitOfEverything_Nested `protobuf:"bytes,2,rep,name=nested,proto3" json:"nested,omitempty"`
	FloatValue          float32                    `protobuf:"fixed32,3,opt,name=float_value,json=floatValue,proto3" json:"float_value,omitempty"`
	DoubleValue         float64                    `protobuf:"fixed64,4,opt,name=double_value,json=doubleValue,proto3" json:"double_value,omitempty"`
	Int64Value          int64                      `protobuf:"varint,5,opt,name=int64_value,json=int64Value,proto3" json:"int64_value,omitempty"`
	Uint64Value         uint64                     `protobuf:"varint,6,opt,name=uint64_value,json=uint64Value,proto3" json:"uint64_value,omitempty"`
	Int32Value          int32                      `protobuf:"varint,7,opt,name=int32_value,json=int32Value,proto3" json:"int32_value,omitempty"`
	Fixed64Value        uint64                     `protobuf:"fixed64,8,opt,name=fixed64_value,json=fixed64Value,proto3" json:"fixed64_value,omitempty"`
	Fixed32Value        uint32                     `protobuf:"fixed32,9,opt,name=fixed32_value,json=fixed32Value,proto3" json:"fixed32_value,omitempty"`
	BoolValue           bool                       `protobuf:"varint,10,opt,name=bool_value,json=boolValue,proto3" json:"bool_value,omitempty"`
	StringValue         string                     `protobuf:"bytes,11,opt,name=string_value,json=stringValue,proto3" json:"string_value,omitempty"`
	BytesValue          []byte                     `protobuf:"bytes,29,opt,name=bytes_value,json=bytesValue,proto3" json:"bytes_value,omitempty"`
	Uint32Value         uint32                     `protobuf:"varint,13,opt,name=uint32_value,json=uint32Value,proto3" json:"uint32_value,omitempty"`
	EnumValue           NumericEnum                `protobuf:"varint,14,opt,name=enum_value,json=enumValue,proto3,enum=grpc.gateway.runtime.internal.examplepb.NumericEnum" json:"enum_value,omitempty"`
	Sfixed32Value       int32                      `protobuf:"fixed32,15,opt,name=sfixed32_value,json=sfixed32Value,proto3" json:"sfixed32_value,omitempty"`
	Sfixed64Value       int64                      `protobuf:"fixed64,16,opt,name=sfixed64_value,json=sfixed64Value,proto3" json:"sfixed64_value,omitempty"`
	Sint32Value         int32                      `protobuf:"zigzag32,17,opt,name=sint32_value,json=sint32Value,proto3" json:"sint32_value,omitempty"`
	Sint64Value         int64                      `protobuf:"zigzag64,18,opt,name=sint64_value,json=sint64Value,proto3" json:"sint64_value,omitempty"`
	RepeatedStringValue []string                   `protobuf:"bytes,19,rep,name=repeated_string_value,json=repeatedStringValue,proto3" json:"repeated_string_value,omitempty"`
	// Types that are valid to be assigned to OneofValue:
	//	*ABitOfEverything_OneofEmpty
	//	*ABitOfEverything_OneofString
	OneofValue               isABitOfEverything_OneofValue       `protobuf_oneof:"oneof_value"`
	MapValue                 map[string]NumericEnum              `protobuf:"bytes,22,rep,name=map_value,json=mapValue,proto3" json:"map_value,omitempty" protobuf_key:"bytes,1,opt,name=key,proto3" protobuf_val:"varint,2,opt,name=value,proto3,enum=grpc.gateway.runtime.internal.examplepb.NumericEnum"`
	MappedStringValue        map[string]string                   `protobuf:"bytes,23,rep,name=mapped_string_value,json=mappedStringValue,proto3" json:"mapped_string_value,omitempty" protobuf_key:"bytes,1,opt,name=key,proto3" protobuf_val:"bytes,2,opt,name=value,proto3"`
	MappedNestedValue        map[string]*ABitOfEverything_Nested `protobuf:"bytes,24,rep,name=mapped_nested_value,json=mappedNestedValue,proto3" json:"mapped_nested_value,omitempty" protobuf_key:"bytes,1,opt,name=key,proto3" protobuf_val:"bytes,2,opt,name=value,proto3"`
	NonConventionalNameValue string                              `protobuf:"bytes,26,opt,name=nonConventionalNameValue,proto3" json:"nonConventionalNameValue,omitempty"`
	TimestampValue           *timestamp.Timestamp                `protobuf:"bytes,27,opt,name=timestamp_value,json=timestampValue,proto3" json:"timestamp_value,omitempty"`
	// repeated enum value. it is comma-separated in query
	RepeatedEnumValue []NumericEnum `protobuf:"varint,28,rep,packed,name=repeated_enum_value,json=repeatedEnumValue,proto3,enum=grpc.gateway.runtime.internal.examplepb.NumericEnum" json:"repeated_enum_value,omitempty"`
	// repeated numeric enum comment (This comment is overridden by the field annotation)
	RepeatedEnumAnnotation []NumericEnum `protobuf:"varint,32,rep,packed,name=repeated_enum_annotation,json=repeatedEnumAnnotation,proto3,enum=grpc.gateway.runtime.internal.examplepb.NumericEnum" json:"repeated_enum_annotation,omitempty"`
	// numeric enum comment (This comment is overridden by the field annotation)
	EnumValueAnnotation NumericEnum `protobuf:"varint,33,opt,name=enum_value_annotation,json=enumValueAnnotation,proto3,enum=grpc.gateway.runtime.internal.examplepb.NumericEnum" json:"enum_value_annotation,omitempty"`
	// repeated string comment (This comment is overridden by the field annotation)
	RepeatedStringAnnotation []string `protobuf:"bytes,34,rep,name=repeated_string_annotation,json=repeatedStringAnnotation,proto3" json:"repeated_string_annotation,omitempty"`
	// repeated nested object comment (This comment is overridden by the field annotation)
	RepeatedNestedAnnotation []*ABitOfEverything_Nested `protobuf:"bytes,35,rep,name=repeated_nested_annotation,json=repeatedNestedAnnotation,proto3" json:"repeated_nested_annotation,omitempty"`
	// nested object comments (This comment is overridden by the field annotation)
	NestedAnnotation     *ABitOfEverything_Nested `protobuf:"bytes,36,opt,name=nested_annotation,json=nestedAnnotation,proto3" json:"nested_annotation,omitempty"`
	Int64OverrideType    int64                    `protobuf:"varint,37,opt,name=int64_override_type,json=int64OverrideType,proto3" json:"int64_override_type,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                 `json:"-"`
	XXX_unrecognized     []byte                   `json:"-"`
	XXX_sizecache        int32                    `json:"-"`
}

func (m *ABitOfEverything) Reset()         { *m = ABitOfEverything{} }
func (m *ABitOfEverything) String() string { return proto.CompactTextString(m) }
func (*ABitOfEverything) ProtoMessage()    {}
func (*ABitOfEverything) Descriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{1}
}

func (m *ABitOfEverything) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ABitOfEverything.Unmarshal(m, b)
}
func (m *ABitOfEverything) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ABitOfEverything.Marshal(b, m, deterministic)
}
func (m *ABitOfEverything) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ABitOfEverything.Merge(m, src)
}
func (m *ABitOfEverything) XXX_Size() int {
	return xxx_messageInfo_ABitOfEverything.Size(m)
}
func (m *ABitOfEverything) XXX_DiscardUnknown() {
	xxx_messageInfo_ABitOfEverything.DiscardUnknown(m)
}

var xxx_messageInfo_ABitOfEverything proto.InternalMessageInfo

func (m *ABitOfEverything) GetSingleNested() *ABitOfEverything_Nested {
	if m != nil {
		return m.SingleNested
	}
	return nil
}

func (m *ABitOfEverything) GetUuid() string {
	if m != nil {
		return m.Uuid
	}
	return ""
}

func (m *ABitOfEverything) GetNested() []*ABitOfEverything_Nested {
	if m != nil {
		return m.Nested
	}
	return nil
}

func (m *ABitOfEverything) GetFloatValue() float32 {
	if m != nil {
		return m.FloatValue
	}
	return 0
}

func (m *ABitOfEverything) GetDoubleValue() float64 {
	if m != nil {
		return m.DoubleValue
	}
	return 0
}

func (m *ABitOfEverything) GetInt64Value() int64 {
	if m != nil {
		return m.Int64Value
	}
	return 0
}

func (m *ABitOfEverything) GetUint64Value() uint64 {
	if m != nil {
		return m.Uint64Value
	}
	return 0
}

func (m *ABitOfEverything) GetInt32Value() int32 {
	if m != nil {
		return m.Int32Value
	}
	return 0
}

func (m *ABitOfEverything) GetFixed64Value() uint64 {
	if m != nil {
		return m.Fixed64Value
	}
	return 0
}

func (m *ABitOfEverything) GetFixed32Value() uint32 {
	if m != nil {
		return m.Fixed32Value
	}
	return 0
}

func (m *ABitOfEverything) GetBoolValue() bool {
	if m != nil {
		return m.BoolValue
	}
	return false
}

func (m *ABitOfEverything) GetStringValue() string {
	if m != nil {
		return m.StringValue
	}
	return ""
}

func (m *ABitOfEverything) GetBytesValue() []byte {
	if m != nil {
		return m.BytesValue
	}
	return nil
}

func (m *ABitOfEverything) GetUint32Value() uint32 {
	if m != nil {
		return m.Uint32Value
	}
	return 0
}

func (m *ABitOfEverything) GetEnumValue() NumericEnum {
	if m != nil {
		return m.EnumValue
	}
	return NumericEnum_ZERO
}

func (m *ABitOfEverything) GetSfixed32Value() int32 {
	if m != nil {
		return m.Sfixed32Value
	}
	return 0
}

func (m *ABitOfEverything) GetSfixed64Value() int64 {
	if m != nil {
		return m.Sfixed64Value
	}
	return 0
}

func (m *ABitOfEverything) GetSint32Value() int32 {
	if m != nil {
		return m.Sint32Value
	}
	return 0
}

func (m *ABitOfEverything) GetSint64Value() int64 {
	if m != nil {
		return m.Sint64Value
	}
	return 0
}

func (m *ABitOfEverything) GetRepeatedStringValue() []string {
	if m != nil {
		return m.RepeatedStringValue
	}
	return nil
}

type isABitOfEverything_OneofValue interface {
	isABitOfEverything_OneofValue()
}

type ABitOfEverything_OneofEmpty struct {
	OneofEmpty *empty.Empty `protobuf:"bytes,20,opt,name=oneof_empty,json=oneofEmpty,proto3,oneof"`
}

type ABitOfEverything_OneofString struct {
	OneofString string `protobuf:"bytes,21,opt,name=oneof_string,json=oneofString,proto3,oneof"`
}

func (*ABitOfEverything_OneofEmpty) isABitOfEverything_OneofValue() {}

func (*ABitOfEverything_OneofString) isABitOfEverything_OneofValue() {}

func (m *ABitOfEverything) GetOneofValue() isABitOfEverything_OneofValue {
	if m != nil {
		return m.OneofValue
	}
	return nil
}

func (m *ABitOfEverything) GetOneofEmpty() *empty.Empty {
	if x, ok := m.GetOneofValue().(*ABitOfEverything_OneofEmpty); ok {
		return x.OneofEmpty
	}
	return nil
}

func (m *ABitOfEverything) GetOneofString() string {
	if x, ok := m.GetOneofValue().(*ABitOfEverything_OneofString); ok {
		return x.OneofString
	}
	return ""
}

func (m *ABitOfEverything) GetMapValue() map[string]NumericEnum {
	if m != nil {
		return m.MapValue
	}
	return nil
}

func (m *ABitOfEverything) GetMappedStringValue() map[string]string {
	if m != nil {
		return m.MappedStringValue
	}
	return nil
}

func (m *ABitOfEverything) GetMappedNestedValue() map[string]*ABitOfEverything_Nested {
	if m != nil {
		return m.MappedNestedValue
	}
	return nil
}

func (m *ABitOfEverything) GetNonConventionalNameValue() string {
	if m != nil {
		return m.NonConventionalNameValue
	}
	return ""
}

func (m *ABitOfEverything) GetTimestampValue() *timestamp.Timestamp {
	if m != nil {
		return m.TimestampValue
	}
	return nil
}

func (m *ABitOfEverything) GetRepeatedEnumValue() []NumericEnum {
	if m != nil {
		return m.RepeatedEnumValue
	}
	return nil
}

func (m *ABitOfEverything) GetRepeatedEnumAnnotation() []NumericEnum {
	if m != nil {
		return m.RepeatedEnumAnnotation
	}
	return nil
}

func (m *ABitOfEverything) GetEnumValueAnnotation() NumericEnum {
	if m != nil {
		return m.EnumValueAnnotation
	}
	return NumericEnum_ZERO
}

func (m *ABitOfEverything) GetRepeatedStringAnnotation() []string {
	if m != nil {
		return m.RepeatedStringAnnotation
	}
	return nil
}

func (m *ABitOfEverything) GetRepeatedNestedAnnotation() []*ABitOfEverything_Nested {
	if m != nil {
		return m.RepeatedNestedAnnotation
	}
	return nil
}

func (m *ABitOfEverything) GetNestedAnnotation() *ABitOfEverything_Nested {
	if m != nil {
		return m.NestedAnnotation
	}
	return nil
}

func (m *ABitOfEverything) GetInt64OverrideType() int64 {
	if m != nil {
		return m.Int64OverrideType
	}
	return 0
}

// XXX_OneofWrappers is for the internal use of the proto package.
func (*ABitOfEverything) XXX_OneofWrappers() []interface{} {
	return []interface{}{
		(*ABitOfEverything_OneofEmpty)(nil),
		(*ABitOfEverything_OneofString)(nil),
	}
}

// Nested is nested type.
type ABitOfEverything_Nested struct {
	// name is nested field.
	Name   string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Amount uint32 `protobuf:"varint,2,opt,name=amount,proto3" json:"amount,omitempty"`
	// DeepEnum comment.
	Ok                   ABitOfEverything_Nested_DeepEnum `protobuf:"varint,3,opt,name=ok,proto3,enum=grpc.gateway.runtime.internal.examplepb.ABitOfEverything_Nested_DeepEnum" json:"ok,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                         `json:"-"`
	XXX_unrecognized     []byte                           `json:"-"`
	XXX_sizecache        int32                            `json:"-"`
}

func (m *ABitOfEverything_Nested) Reset()         { *m = ABitOfEverything_Nested{} }
func (m *ABitOfEverything_Nested) String() string { return proto.CompactTextString(m) }
func (*ABitOfEverything_Nested) ProtoMessage()    {}
func (*ABitOfEverything_Nested) Descriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{1, 0}
}

func (m *ABitOfEverything_Nested) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ABitOfEverything_Nested.Unmarshal(m, b)
}
func (m *ABitOfEverything_Nested) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ABitOfEverything_Nested.Marshal(b, m, deterministic)
}
func (m *ABitOfEverything_Nested) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ABitOfEverything_Nested.Merge(m, src)
}
func (m *ABitOfEverything_Nested) XXX_Size() int {
	return xxx_messageInfo_ABitOfEverything_Nested.Size(m)
}
func (m *ABitOfEverything_Nested) XXX_DiscardUnknown() {
	xxx_messageInfo_ABitOfEverything_Nested.DiscardUnknown(m)
}

var xxx_messageInfo_ABitOfEverything_Nested proto.InternalMessageInfo

func (m *ABitOfEverything_Nested) GetName() string {
	if m != nil {
		return m.Name
	}
	return ""
}

func (m *ABitOfEverything_Nested) GetAmount() uint32 {
	if m != nil {
		return m.Amount
	}
	return 0
}

func (m *ABitOfEverything_Nested) GetOk() ABitOfEverything_Nested_DeepEnum {
	if m != nil {
		return m.Ok
	}
	return ABitOfEverything_Nested_FALSE
}

type ResponseBodyOut struct {
	Response             *ResponseBodyOut_Response `protobuf:"bytes,2,opt,name=response,proto3" json:"response,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                  `json:"-"`
	XXX_unrecognized     []byte                    `json:"-"`
	XXX_sizecache        int32                     `json:"-"`
}

func (m *ResponseBodyOut) Reset()         { *m = ResponseBodyOut{} }
func (m *ResponseBodyOut) String() string { return proto.CompactTextString(m) }
func (*ResponseBodyOut) ProtoMessage()    {}
func (*ResponseBodyOut) Descriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{2}
}

func (m *ResponseBodyOut) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ResponseBodyOut.Unmarshal(m, b)
}
func (m *ResponseBodyOut) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ResponseBodyOut.Marshal(b, m, deterministic)
}
func (m *ResponseBodyOut) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ResponseBodyOut.Merge(m, src)
}
func (m *ResponseBodyOut) XXX_Size() int {
	return xxx_messageInfo_ResponseBodyOut.Size(m)
}
func (m *ResponseBodyOut) XXX_DiscardUnknown() {
	xxx_messageInfo_ResponseBodyOut.DiscardUnknown(m)
}

var xxx_messageInfo_ResponseBodyOut proto.InternalMessageInfo

func (m *ResponseBodyOut) GetResponse() *ResponseBodyOut_Response {
	if m != nil {
		return m.Response
	}
	return nil
}

type ResponseBodyOut_Response struct {
	Data                 string   `protobuf:"bytes,1,opt,name=data,proto3" json:"data,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *ResponseBodyOut_Response) Reset()         { *m = ResponseBodyOut_Response{} }
func (m *ResponseBodyOut_Response) String() string { return proto.CompactTextString(m) }
func (*ResponseBodyOut_Response) ProtoMessage()    {}
func (*ResponseBodyOut_Response) Descriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{2, 0}
}

func (m *ResponseBodyOut_Response) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ResponseBodyOut_Response.Unmarshal(m, b)
}
func (m *ResponseBodyOut_Response) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ResponseBodyOut_Response.Marshal(b, m, deterministic)
}
func (m *ResponseBodyOut_Response) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ResponseBodyOut_Response.Merge(m, src)
}
func (m *ResponseBodyOut_Response) XXX_Size() int {
	return xxx_messageInfo_ResponseBodyOut_Response.Size(m)
}
func (m *ResponseBodyOut_Response) XXX_DiscardUnknown() {
	xxx_messageInfo_ResponseBodyOut_Response.DiscardUnknown(m)
}

var xxx_messageInfo_ResponseBodyOut_Response proto.InternalMessageInfo

func (m *ResponseBodyOut_Response) GetData() string {
	if m != nil {
		return m.Data
	}
	return ""
}

type RepeatedResponseBodyOut struct {
	Response             []*RepeatedResponseBodyOut_Response `protobuf:"bytes,2,rep,name=response,proto3" json:"response,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                            `json:"-"`
	XXX_unrecognized     []byte                              `json:"-"`
	XXX_sizecache        int32                               `json:"-"`
}

func (m *RepeatedResponseBodyOut) Reset()         { *m = RepeatedResponseBodyOut{} }
func (m *RepeatedResponseBodyOut) String() string { return proto.CompactTextString(m) }
func (*RepeatedResponseBodyOut) ProtoMessage()    {}
func (*RepeatedResponseBodyOut) Descriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{3}
}

func (m *RepeatedResponseBodyOut) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_RepeatedResponseBodyOut.Unmarshal(m, b)
}
func (m *RepeatedResponseBodyOut) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_RepeatedResponseBodyOut.Marshal(b, m, deterministic)
}
func (m *RepeatedResponseBodyOut) XXX_Merge(src proto.Message) {
	xxx_messageInfo_RepeatedResponseBodyOut.Merge(m, src)
}
func (m *RepeatedResponseBodyOut) XXX_Size() int {
	return xxx_messageInfo_RepeatedResponseBodyOut.Size(m)
}
func (m *RepeatedResponseBodyOut) XXX_DiscardUnknown() {
	xxx_messageInfo_RepeatedResponseBodyOut.DiscardUnknown(m)
}

var xxx_messageInfo_RepeatedResponseBodyOut proto.InternalMessageInfo

func (m *RepeatedResponseBodyOut) GetResponse() []*RepeatedResponseBodyOut_Response {
	if m != nil {
		return m.Response
	}
	return nil
}

type RepeatedResponseBodyOut_Response struct {
	Data                 string                                        `protobuf:"bytes,1,opt,name=data,proto3" json:"data,omitempty"`
	Type                 RepeatedResponseBodyOut_Response_ResponseType `protobuf:"varint,3,opt,name=type,proto3,enum=grpc.gateway.runtime.internal.examplepb.RepeatedResponseBodyOut_Response_ResponseType" json:"type,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                                      `json:"-"`
	XXX_unrecognized     []byte                                        `json:"-"`
	XXX_sizecache        int32                                         `json:"-"`
}

func (m *RepeatedResponseBodyOut_Response) Reset()         { *m = RepeatedResponseBodyOut_Response{} }
func (m *RepeatedResponseBodyOut_Response) String() string { return proto.CompactTextString(m) }
func (*RepeatedResponseBodyOut_Response) ProtoMessage()    {}
func (*RepeatedResponseBodyOut_Response) Descriptor() ([]byte, []int) {
	return fileDescriptor_411aaf1bcbdb81d6, []int{3, 0}
}

func (m *RepeatedResponseBodyOut_Response) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_RepeatedResponseBodyOut_Response.Unmarshal(m, b)
}
func (m *RepeatedResponseBodyOut_Response) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_RepeatedResponseBodyOut_Response.Marshal(b, m, deterministic)
}
func (m *RepeatedResponseBodyOut_Response) XXX_Merge(src proto.Message) {
	xxx_messageInfo_RepeatedResponseBodyOut_Response.Merge(m, src)
}
func (m *RepeatedResponseBodyOut_Response) XXX_Size() int {
	return xxx_messageInfo_RepeatedResponseBodyOut_Response.Size(m)
}
func (m *RepeatedResponseBodyOut_Response) XXX_DiscardUnknown() {
	xxx_messageInfo_RepeatedResponseBodyOut_Response.DiscardUnknown(m)
}

var xxx_messageInfo_RepeatedResponseBodyOut_Response proto.InternalMessageInfo

func (m *RepeatedResponseBodyOut_Response) GetData() string {
	if m != nil {
		return m.Data
	}
	return ""
}

func (m *RepeatedResponseBodyOut_Response) GetType() RepeatedResponseBodyOut_Response_ResponseType {
	if m != nil {
		return m.Type
	}
	return RepeatedResponseBodyOut_Response_UNKNOWN
}

func init() {
	proto.RegisterEnum("grpc.gateway.runtime.internal.examplepb.NumericEnum", NumericEnum_name, NumericEnum_value)
	proto.RegisterEnum("grpc.gateway.runtime.internal.examplepb.ABitOfEverything_Nested_DeepEnum", ABitOfEverything_Nested_DeepEnum_name, ABitOfEverything_Nested_DeepEnum_value)
	proto.RegisterEnum("grpc.gateway.runtime.internal.examplepb.RepeatedResponseBodyOut_Response_ResponseType", RepeatedResponseBodyOut_Response_ResponseType_name, RepeatedResponseBodyOut_Response_ResponseType_value)
	proto.RegisterType((*SimpleMessage)(nil), "grpc.gateway.runtime.internal.examplepb.SimpleMessage")
	proto.RegisterType((*ABitOfEverything)(nil), "grpc.gateway.runtime.internal.examplepb.ABitOfEverything")
	proto.RegisterMapType((map[string]NumericEnum)(nil), "grpc.gateway.runtime.internal.examplepb.ABitOfEverything.MapValueEntry")
	proto.RegisterMapType((map[string]*ABitOfEverything_Nested)(nil), "grpc.gateway.runtime.internal.examplepb.ABitOfEverything.MappedNestedValueEntry")
	proto.RegisterMapType((map[string]string)(nil), "grpc.gateway.runtime.internal.examplepb.ABitOfEverything.MappedStringValueEntry")
	proto.RegisterType((*ABitOfEverything_Nested)(nil), "grpc.gateway.runtime.internal.examplepb.ABitOfEverything.Nested")
	proto.RegisterType((*ResponseBodyOut)(nil), "grpc.gateway.runtime.internal.examplepb.ResponseBodyOut")
	proto.RegisterType((*ResponseBodyOut_Response)(nil), "grpc.gateway.runtime.internal.examplepb.ResponseBodyOut.Response")
	proto.RegisterType((*RepeatedResponseBodyOut)(nil), "grpc.gateway.runtime.internal.examplepb.RepeatedResponseBodyOut")
	proto.RegisterType((*RepeatedResponseBodyOut_Response)(nil), "grpc.gateway.runtime.internal.examplepb.RepeatedResponseBodyOut.Response")
}

func init() {
	proto.RegisterFile("runtime/internal/examplepb/example.proto", fileDescriptor_411aaf1bcbdb81d6)
}

var fileDescriptor_411aaf1bcbdb81d6 = []byte{
	// 1101 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xac, 0x55, 0x5d, 0x73, 0x1b, 0x35,
	0x1b, 0x8d, 0xec, 0xc4, 0xb1, 0x9f, 0xb5, 0x9d, 0xb5, 0xf2, 0xd1, 0x7d, 0xdd, 0xb7, 0x64, 0xeb,
	0xd0, 0x61, 0xe1, 0x62, 0x33, 0xe3, 0x74, 0x18, 0xe8, 0x70, 0x41, 0xd2, 0x1a, 0x5a, 0xa0, 0x36,
	0xa3, 0xa4, 0x01, 0x3a, 0xc3, 0x78, 0xd6, 0xb5, 0xe2, 0x2e, 0xf1, 0x6a, 0x97, 0xfd, 0x08, 0xf5,
	0x0d, 0xc3, 0x15, 0xd7, 0xfc, 0x14, 0x6e, 0x98, 0xe1, 0x3f, 0xf1, 0x27, 0x98, 0x95, 0xb4, 0x8a,
	0xec, 0x04, 0x26, 0x34, 0xbe, 0x5a, 0xe9, 0xd1, 0xd1, 0x39, 0x47, 0x5a, 0x1d, 0x09, 0x9c, 0x38,
	0x63, 0xa9, 0x1f, 0xd0, 0x7d, 0x9f, 0xa5, 0x34, 0x66, 0xde, 0x74, 0x9f, 0xbe, 0xf1, 0x82, 0x68,
	0x4a, 0xa3, 0x51, 0xd1, 0x72, 0xa3, 0x38, 0x4c, 0x43, 0xfc, 0xde, 0x24, 0x8e, 0x5e, 0xb9, 0x13,
	0x2f, 0xa5, 0x3f, 0x79, 0x33, 0x57, 0x4e, 0x73, 0x8b, 0x69, 0xae, 0x9a, 0xd6, 0xde, 0x9d, 0x84,
	0xe1, 0x64, 0x4a, 0xf7, 0xf9, 0xb4, 0x51, 0x76, 0xb6, 0x9f, 0x03, 0x93, 0xd4, 0x0b, 0x22, 0xc1,
	0xd4, 0xbe, 0xbb, 0x08, 0xa0, 0x41, 0x94, 0xce, 0xc4, 0x60, 0x67, 0x17, 0x1a, 0xc7, 0x7e, 0xce,
	0xf4, 0x9c, 0x26, 0x89, 0x37, 0xa1, 0xb8, 0x09, 0x25, 0x7f, 0x6c, 0x21, 0x1b, 0x39, 0x35, 0x52,
	0xf2, 0xc7, 0x9d, 0xbf, 0xb6, 0xc0, 0x3c, 0x3c, 0xf2, 0xd3, 0xc1, 0x59, 0xef, 0x82, 0xc6, 0xb3,
	0xf4, 0xb5, 0xcf, 0x26, 0x98, 0x42, 0x23, 0xf1, 0xd9, 0x64, 0x4a, 0x87, 0x8c, 0x26, 0x29, 0x1d,
	0x5b, 0xff, 0xb3, 0x91, 0x63, 0x74, 0x3f, 0x75, 0x6f, 0x68, 0xda, 0x5d, 0x64, 0x74, 0xfb, 0x9c,
	0x87, 0xd4, 0x05, 0xad, 0xe8, 0x61, 0x0c, 0xab, 0x59, 0xa6, 0xdc, 0xf0, 0x36, 0xfe, 0x16, 0x2a,
	0x52, 0xb3, 0x64, 0x97, 0x97, 0xa2, 0x29, 0xf9, 0xf0, 0x2e, 0x18, 0x67, 0xd3, 0xd0, 0x4b, 0x87,
	0x17, 0xde, 0x34, 0xa3, 0x56, 0xd9, 0x46, 0x4e, 0x89, 0x00, 0x2f, 0x9d, 0xe6, 0x15, 0x7c, 0x1f,
	0xea, 0xe3, 0x30, 0x1b, 0x4d, 0xa9, 0x44, 0xac, 0xda, 0xc8, 0x41, 0xc4, 0x10, 0x35, 0x01, 0xd9,
	0x05, 0xc3, 0x67, 0xe9, 0x87, 0x0f, 0x25, 0x62, 0xcd, 0x46, 0x4e, 0x99, 0x00, 0x2f, 0x29, 0x8e,
	0x4c, 0x47, 0x54, 0x6c, 0xe4, 0xac, 0x12, 0x23, 0xd3, 0x20, 0x82, 0xe3, 0xa0, 0x2b, 0x11, 0xeb,
	0x36, 0x72, 0xd6, 0x38, 0xc7, 0x41, 0x57, 0x00, 0xf6, 0xa0, 0x71, 0xe6, 0xbf, 0xa1, 0x63, 0x45,
	0x52, 0xb5, 0x91, 0x53, 0x21, 0x75, 0x59, 0x9c, 0x07, 0x29, 0x9e, 0x9a, 0x8d, 0x9c, 0x75, 0x09,
	0x2a, 0x98, 0xee, 0x01, 0x8c, 0xc2, 0x70, 0x2a, 0x11, 0x60, 0x23, 0xa7, 0x4a, 0x6a, 0x79, 0x45,
	0x99, 0x4d, 0xd2, 0xd8, 0x67, 0x13, 0x09, 0x30, 0xf8, 0x7f, 0x30, 0x44, 0x4d, 0x99, 0x1d, 0xcd,
	0x52, 0x9a, 0x48, 0xc4, 0x3d, 0x1b, 0x39, 0x75, 0x02, 0xbc, 0x34, 0xb7, 0x60, 0x65, 0xa3, 0x61,
	0x23, 0xa7, 0x21, 0x16, 0x5c, 0xb8, 0x38, 0x06, 0xa0, 0x2c, 0x0b, 0x24, 0xa0, 0x69, 0x23, 0xa7,
	0xd9, 0x7d, 0x78, 0xe3, 0xdf, 0xda, 0xcf, 0x02, 0x1a, 0xfb, 0xaf, 0x7a, 0x2c, 0x0b, 0x48, 0x2d,
	0xe7, 0x11, 0xa4, 0x0f, 0xa0, 0x99, 0xcc, 0x6f, 0xc0, 0x86, 0x8d, 0x9c, 0x0d, 0xd2, 0x48, 0xe6,
	0x76, 0x40, 0xc1, 0xd4, 0x66, 0x9a, 0x36, 0x72, 0xcc, 0x02, 0xa6, 0xfd, 0xb6, 0x44, 0x5f, 0x45,
	0xcb, 0x46, 0x4e, 0x8b, 0x18, 0x89, 0xb6, 0x0a, 0x09, 0x51, 0x3c, 0xd8, 0x46, 0x0e, 0x16, 0x90,
	0x82, 0xa5, 0x0b, 0xdb, 0x31, 0x8d, 0xa8, 0x97, 0xd2, 0xf1, 0x70, 0x6e, 0x63, 0x37, 0xed, 0xb2,
	0x53, 0x23, 0x9b, 0xc5, 0xe0, 0xb1, 0xb6, 0xc1, 0x1f, 0x83, 0x11, 0x32, 0x1a, 0x9e, 0x0d, 0x79,
	0x6a, 0xad, 0x2d, 0x1e, 0xb4, 0x1d, 0x57, 0x64, 0xda, 0x2d, 0x32, 0xed, 0xf6, 0xf2, 0xd1, 0xa7,
	0x2b, 0x04, 0x38, 0x98, 0xf7, 0xf0, 0x1e, 0xd4, 0xc5, 0x54, 0xa1, 0x65, 0x6d, 0xe7, 0xbf, 0xef,
	0xe9, 0x0a, 0x11, 0x84, 0x42, 0x04, 0x8f, 0xa1, 0x16, 0x78, 0x91, 0xf4, 0xb1, 0xc3, 0x23, 0xf5,
	0xf9, 0xdb, 0x47, 0xea, 0xb9, 0x17, 0x71, 0xdb, 0x3d, 0x96, 0xc6, 0x33, 0x52, 0x0d, 0x64, 0x17,
	0xff, 0x82, 0x60, 0x33, 0xf0, 0xa2, 0x68, 0x71, 0xe1, 0x77, 0xb8, 0xe0, 0xd7, 0xb7, 0x12, 0x8c,
	0xe6, 0x36, 0x4c, 0x28, 0xb7, 0x82, 0xc5, 0xba, 0x6e, 0x41, 0x04, 0x5e, 0x5a, 0xb0, 0x96, 0x63,
	0x41, 0x5c, 0x26, 0x57, 0x2d, 0x68, 0x75, 0xfc, 0x08, 0x2c, 0x16, 0xb2, 0xc7, 0x21, 0xbb, 0xa0,
	0x2c, 0xf5, 0x43, 0xe6, 0x4d, 0xfb, 0x5e, 0x20, 0x6e, 0x0e, 0xab, 0xcd, 0xb3, 0xf5, 0x8f, 0xe3,
	0xf8, 0x31, 0x6c, 0xa8, 0x8b, 0x5d, 0x3a, 0xbf, 0xcb, 0xcf, 0x42, 0xfb, 0xca, 0x59, 0x38, 0x29,
	0x70, 0xa4, 0xa9, 0xa6, 0x08, 0x92, 0x31, 0xa8, 0x33, 0x36, 0xd4, 0x22, 0xf7, 0x7f, 0xbb, 0xfc,
	0xd6, 0x91, 0x6b, 0x15, 0x84, 0x3d, 0x15, 0x3d, 0x06, 0xd6, 0xbc, 0x8a, 0xc7, 0x58, 0x98, 0x7a,
	0xf9, 0x8a, 0x2c, 0xfb, 0x16, 0x52, 0x3b, 0xba, 0xd4, 0xa1, 0xe2, 0xc4, 0xaf, 0x61, 0xfb, 0x72,
	0x31, 0xba, 0xd8, 0xfd, 0x5b, 0x5c, 0x25, 0x9b, 0xea, 0x2a, 0xd1, 0x94, 0x3e, 0x81, 0xf6, 0x62,
	0x80, 0x35, 0xb9, 0x0e, 0x4f, 0xb1, 0x35, 0x9f, 0x62, 0x6d, 0xf6, 0xcf, 0xda, 0x6c, 0x79, 0x04,
	0xb5, 0xd9, 0x7b, 0x4b, 0x7a, 0xce, 0x94, 0xbe, 0xe8, 0x6b, 0xfa, 0x01, 0xb4, 0xae, 0xca, 0xbe,
	0xbb, 0xa4, 0x97, 0xdb, 0x64, 0x8b, 0x72, 0x2e, 0x6c, 0x8a, 0xfb, 0x30, 0xbc, 0xa0, 0x71, 0xec,
	0x8f, 0xe9, 0x30, 0x9d, 0x45, 0xd4, 0x7a, 0xc0, 0xdf, 0xc4, 0x16, 0x1f, 0x1a, 0xc8, 0x91, 0x93,
	0x59, 0x44, 0xdb, 0xbf, 0x23, 0xa8, 0x5c, 0x3e, 0xfc, 0xcc, 0x0b, 0x68, 0xf1, 0xf0, 0xe7, 0x6d,
	0xbc, 0x03, 0x15, 0x2f, 0x08, 0x33, 0x96, 0x5a, 0x25, 0xfe, 0x84, 0xc8, 0x1e, 0xfe, 0x0e, 0x4a,
	0xe1, 0x39, 0x7f, 0xad, 0x9b, 0xdd, 0x67, 0xb7, 0x5d, 0x86, 0xfb, 0x84, 0xd2, 0x88, 0xff, 0xff,
	0x52, 0x78, 0xde, 0xd9, 0x85, 0x6a, 0xd1, 0xc7, 0x35, 0x58, 0xfb, 0xec, 0xf0, 0xab, 0xe3, 0x9e,
	0xb9, 0x82, 0xab, 0xb0, 0x7a, 0x42, 0x5e, 0xf4, 0x4c, 0xd4, 0xfe, 0x11, 0x1a, 0x73, 0x37, 0x1e,
	0x36, 0xa1, 0x7c, 0x4e, 0x67, 0xd2, 0x77, 0xde, 0xc4, 0x5f, 0xc0, 0x9a, 0x08, 0x59, 0xe9, 0x16,
	0x87, 0x51, 0x50, 0x3c, 0x2a, 0x7d, 0x84, 0xda, 0x4f, 0x60, 0xe7, 0xfa, 0x3b, 0xef, 0x1a, 0xed,
	0x2d, 0x5d, 0xbb, 0xa6, 0xb3, 0xfc, 0x8a, 0x0a, 0x9a, 0xc5, 0x7b, 0xeb, 0x1a, 0x9a, 0x53, 0x9d,
	0x66, 0x19, 0x67, 0xe5, 0xd2, 0xc8, 0x51, 0xa3, 0x78, 0xde, 0x78, 0xa9, 0xf3, 0x1b, 0x82, 0x0d,
	0x42, 0x93, 0x28, 0x64, 0x09, 0x3d, 0x0a, 0xc7, 0xb3, 0x41, 0x96, 0xe2, 0xef, 0xa1, 0x1a, 0xcb,
	0x92, 0x74, 0x70, 0x78, 0x63, 0x07, 0x0b, 0x5c, 0xaa, 0x4f, 0x14, 0x65, 0xfb, 0x1d, 0xa8, 0x16,
	0xd5, 0xfc, 0xdc, 0x8d, 0xbd, 0xd4, 0x2b, 0xce, 0x5d, 0xde, 0xee, 0xfc, 0x59, 0x82, 0x3b, 0x44,
	0x46, 0x6a, 0xd1, 0x1a, 0x9d, 0xb3, 0x96, 0xe7, 0xf7, 0xd9, 0x7f, 0xb0, 0x76, 0x2d, 0xe7, 0x75,
	0x16, 0xff, 0x40, 0xff, 0xee, 0x11, 0xff, 0x00, 0xab, 0x3c, 0x5b, 0x22, 0x05, 0xa7, 0x4b, 0xf3,
	0xa0, 0x1a, 0x79, 0x40, 0x09, 0xd7, 0xe8, 0xbc, 0x0f, 0x75, 0xbd, 0x8a, 0x0d, 0x58, 0x7f, 0xd1,
	0xff, 0xb2, 0x3f, 0xf8, 0xa6, 0x6f, 0xae, 0xe0, 0x35, 0x40, 0x87, 0x26, 0xca, 0x3f, 0x47, 0x66,
	0xe9, 0x03, 0x1b, 0x0c, 0xed, 0x14, 0xe7, 0xb9, 0x79, 0xd9, 0x23, 0x03, 0x73, 0x05, 0xaf, 0x43,
	0x79, 0xd0, 0xef, 0x99, 0xe8, 0xc8, 0x78, 0x59, 0x53, 0x6e, 0x46, 0x15, 0xfe, 0x82, 0x1d, 0xfc,
	0x1d, 0x00, 0x00, 0xff, 0xff, 0xbb, 0xc2, 0xf3, 0x9b, 0x25, 0x0d, 0x00, 0x00,
}
