syntax = "proto3";

package grpc.gateway.protoc_gen_swagger.options;

option go_package = "github.com/grpc-ecosystem/grpc-gateway/protoc-gen-swagger/options";

import "google/protobuf/descriptor.proto";
import "protoc-gen-swagger/options/openapiv2.proto";

extend google.protobuf.FileOptions {
  // ID assigned by protobuf-global-extension-registry@google.com for grpc-gateway project.
  //
  // All IDs are the same, as assigned. It is okay that they are the same, as they extend
  // different descriptor messages.
  Swagger openapiv2_swagger = 1042;
}
extend google.protobuf.MethodOptions {
  // ID assigned by protobuf-global-extension-registry@google.com for grpc-gateway project.
  //
  // All IDs are the same, as assigned. It is okay that they are the same, as they extend
  // different descriptor messages.
  Operation openapiv2_operation = 1042;
}
extend google.protobuf.MessageOptions {
  // ID assigned by protobuf-global-extension-registry@google.com for grpc-gateway project.
  //
  // All IDs are the same, as assigned. It is okay that they are the same, as they extend
  // different descriptor messages.
  Schema openapiv2_schema = 1042;
}
extend google.protobuf.ServiceOptions {
  // ID assigned by protobuf-global-extension-registry@google.com for grpc-gateway project.
  //
  // All IDs are the same, as assigned. It is okay that they are the same, as they extend
  // different descriptor messages.
  Tag openapiv2_tag = 1042;
}
extend google.protobuf.FieldOptions {
  // ID assigned by protobuf-global-extension-registry@google.com for grpc-gateway project.
  //
  // All IDs are the same, as assigned. It is okay that they are the same, as they extend
  // different descriptor messages.
  JSONSchema openapiv2_field = 1042;
}
