/*
 * examples/internal/proto/examplepb/response_body_service.proto
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: version not set
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package responsebody

type ResponseResponseType string

// List of ResponseResponseType
const (
	UNKNOWN_ResponseResponseType ResponseResponseType = "UNKNOWN"
	A_ResponseResponseType ResponseResponseType = "A"
	B_ResponseResponseType ResponseResponseType = "B"
)
