/*
 * A Bit of Everything
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 1.0
 * Contact: none@example.com
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package abe

type ExamplepbABitOfEverythingRepeated struct {
	PathRepeatedFloatValue []float32 `json:"path_repeated_float_value,omitempty"`
	PathRepeatedDoubleValue []float64 `json:"path_repeated_double_value,omitempty"`
	PathRepeatedInt64Value []string `json:"path_repeated_int64_value,omitempty"`
	PathRepeatedUint64Value []string `json:"path_repeated_uint64_value,omitempty"`
	PathRepeatedInt32Value []int32 `json:"path_repeated_int32_value,omitempty"`
	PathRepeatedFixed64Value []string `json:"path_repeated_fixed64_value,omitempty"`
	PathRepeatedFixed32Value []int64 `json:"path_repeated_fixed32_value,omitempty"`
	PathRepeatedBoolValue []bool `json:"path_repeated_bool_value,omitempty"`
	PathRepeatedStringValue []string `json:"path_repeated_string_value,omitempty"`
	PathRepeatedBytesValue []string `json:"path_repeated_bytes_value,omitempty"`
	PathRepeatedUint32Value []int64 `json:"path_repeated_uint32_value,omitempty"`
	PathRepeatedEnumValue []ExamplepbNumericEnum `json:"path_repeated_enum_value,omitempty"`
	PathRepeatedSfixed32Value []int32 `json:"path_repeated_sfixed32_value,omitempty"`
	PathRepeatedSfixed64Value []string `json:"path_repeated_sfixed64_value,omitempty"`
	PathRepeatedSint32Value []int32 `json:"path_repeated_sint32_value,omitempty"`
	PathRepeatedSint64Value []string `json:"path_repeated_sint64_value,omitempty"`
}
