// Code generated by protoc-gen-go. DO NOT EDIT.
// source: examples/internal/proto/examplepb/non_standard_names.proto

package examplepb

import (
	context "context"
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	field_mask "google.golang.org/genproto/protobuf/field_mask"
	grpc "google.golang.org/grpc"
	codes "google.golang.org/grpc/codes"
	status "google.golang.org/grpc/status"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

// NonStandardMessage has oddly named fields.
type NonStandardMessage struct {
	// Id represents the message identifier.
	Id                   string                    `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	Num                  int64                     `protobuf:"varint,2,opt,name=Num,proto3" json:"Num,omitempty"`
	LineNum              int64                     `protobuf:"varint,3,opt,name=line_num,json=lineNum,proto3" json:"line_num,omitempty"`
	LangIdent            string                    `protobuf:"bytes,4,opt,name=langIdent,proto3" json:"langIdent,omitempty"`
	STATUS               string                    `protobuf:"bytes,5,opt,name=STATUS,proto3" json:"STATUS,omitempty"`
	En_GB                int64                     `protobuf:"varint,6,opt,name=en_GB,json=enGB,proto3" json:"en_GB,omitempty"`
	No                   string                    `protobuf:"bytes,7,opt,name=no,proto3" json:"no,omitempty"`
	Thing                *NonStandardMessage_Thing `protobuf:"bytes,8,opt,name=thing,proto3" json:"thing,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                  `json:"-"`
	XXX_unrecognized     []byte                    `json:"-"`
	XXX_sizecache        int32                     `json:"-"`
}

func (m *NonStandardMessage) Reset()         { *m = NonStandardMessage{} }
func (m *NonStandardMessage) String() string { return proto.CompactTextString(m) }
func (*NonStandardMessage) ProtoMessage()    {}
func (*NonStandardMessage) Descriptor() ([]byte, []int) {
	return fileDescriptor_a1fc3d96d9306f6b, []int{0}
}

func (m *NonStandardMessage) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_NonStandardMessage.Unmarshal(m, b)
}
func (m *NonStandardMessage) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_NonStandardMessage.Marshal(b, m, deterministic)
}
func (m *NonStandardMessage) XXX_Merge(src proto.Message) {
	xxx_messageInfo_NonStandardMessage.Merge(m, src)
}
func (m *NonStandardMessage) XXX_Size() int {
	return xxx_messageInfo_NonStandardMessage.Size(m)
}
func (m *NonStandardMessage) XXX_DiscardUnknown() {
	xxx_messageInfo_NonStandardMessage.DiscardUnknown(m)
}

var xxx_messageInfo_NonStandardMessage proto.InternalMessageInfo

func (m *NonStandardMessage) GetId() string {
	if m != nil {
		return m.Id
	}
	return ""
}

func (m *NonStandardMessage) GetNum() int64 {
	if m != nil {
		return m.Num
	}
	return 0
}

func (m *NonStandardMessage) GetLineNum() int64 {
	if m != nil {
		return m.LineNum
	}
	return 0
}

func (m *NonStandardMessage) GetLangIdent() string {
	if m != nil {
		return m.LangIdent
	}
	return ""
}

func (m *NonStandardMessage) GetSTATUS() string {
	if m != nil {
		return m.STATUS
	}
	return ""
}

func (m *NonStandardMessage) GetEn_GB() int64 {
	if m != nil {
		return m.En_GB
	}
	return 0
}

func (m *NonStandardMessage) GetNo() string {
	if m != nil {
		return m.No
	}
	return ""
}

func (m *NonStandardMessage) GetThing() *NonStandardMessage_Thing {
	if m != nil {
		return m.Thing
	}
	return nil
}

type NonStandardMessage_Thing struct {
	SubThing             *NonStandardMessage_Thing_SubThing `protobuf:"bytes,1,opt,name=subThing,proto3" json:"subThing,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                           `json:"-"`
	XXX_unrecognized     []byte                             `json:"-"`
	XXX_sizecache        int32                              `json:"-"`
}

func (m *NonStandardMessage_Thing) Reset()         { *m = NonStandardMessage_Thing{} }
func (m *NonStandardMessage_Thing) String() string { return proto.CompactTextString(m) }
func (*NonStandardMessage_Thing) ProtoMessage()    {}
func (*NonStandardMessage_Thing) Descriptor() ([]byte, []int) {
	return fileDescriptor_a1fc3d96d9306f6b, []int{0, 0}
}

func (m *NonStandardMessage_Thing) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_NonStandardMessage_Thing.Unmarshal(m, b)
}
func (m *NonStandardMessage_Thing) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_NonStandardMessage_Thing.Marshal(b, m, deterministic)
}
func (m *NonStandardMessage_Thing) XXX_Merge(src proto.Message) {
	xxx_messageInfo_NonStandardMessage_Thing.Merge(m, src)
}
func (m *NonStandardMessage_Thing) XXX_Size() int {
	return xxx_messageInfo_NonStandardMessage_Thing.Size(m)
}
func (m *NonStandardMessage_Thing) XXX_DiscardUnknown() {
	xxx_messageInfo_NonStandardMessage_Thing.DiscardUnknown(m)
}

var xxx_messageInfo_NonStandardMessage_Thing proto.InternalMessageInfo

func (m *NonStandardMessage_Thing) GetSubThing() *NonStandardMessage_Thing_SubThing {
	if m != nil {
		return m.SubThing
	}
	return nil
}

type NonStandardMessage_Thing_SubThing struct {
	SubValue             string   `protobuf:"bytes,1,opt,name=sub_value,json=subValue,proto3" json:"sub_value,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *NonStandardMessage_Thing_SubThing) Reset()         { *m = NonStandardMessage_Thing_SubThing{} }
func (m *NonStandardMessage_Thing_SubThing) String() string { return proto.CompactTextString(m) }
func (*NonStandardMessage_Thing_SubThing) ProtoMessage()    {}
func (*NonStandardMessage_Thing_SubThing) Descriptor() ([]byte, []int) {
	return fileDescriptor_a1fc3d96d9306f6b, []int{0, 0, 0}
}

func (m *NonStandardMessage_Thing_SubThing) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_NonStandardMessage_Thing_SubThing.Unmarshal(m, b)
}
func (m *NonStandardMessage_Thing_SubThing) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_NonStandardMessage_Thing_SubThing.Marshal(b, m, deterministic)
}
func (m *NonStandardMessage_Thing_SubThing) XXX_Merge(src proto.Message) {
	xxx_messageInfo_NonStandardMessage_Thing_SubThing.Merge(m, src)
}
func (m *NonStandardMessage_Thing_SubThing) XXX_Size() int {
	return xxx_messageInfo_NonStandardMessage_Thing_SubThing.Size(m)
}
func (m *NonStandardMessage_Thing_SubThing) XXX_DiscardUnknown() {
	xxx_messageInfo_NonStandardMessage_Thing_SubThing.DiscardUnknown(m)
}

var xxx_messageInfo_NonStandardMessage_Thing_SubThing proto.InternalMessageInfo

func (m *NonStandardMessage_Thing_SubThing) GetSubValue() string {
	if m != nil {
		return m.SubValue
	}
	return ""
}

type NonStandardUpdateRequest struct {
	Body                 *NonStandardMessage   `protobuf:"bytes,1,opt,name=body,proto3" json:"body,omitempty"`
	UpdateMask           *field_mask.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	XXX_NoUnkeyedLiteral struct{}              `json:"-"`
	XXX_unrecognized     []byte                `json:"-"`
	XXX_sizecache        int32                 `json:"-"`
}

func (m *NonStandardUpdateRequest) Reset()         { *m = NonStandardUpdateRequest{} }
func (m *NonStandardUpdateRequest) String() string { return proto.CompactTextString(m) }
func (*NonStandardUpdateRequest) ProtoMessage()    {}
func (*NonStandardUpdateRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_a1fc3d96d9306f6b, []int{1}
}

func (m *NonStandardUpdateRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_NonStandardUpdateRequest.Unmarshal(m, b)
}
func (m *NonStandardUpdateRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_NonStandardUpdateRequest.Marshal(b, m, deterministic)
}
func (m *NonStandardUpdateRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_NonStandardUpdateRequest.Merge(m, src)
}
func (m *NonStandardUpdateRequest) XXX_Size() int {
	return xxx_messageInfo_NonStandardUpdateRequest.Size(m)
}
func (m *NonStandardUpdateRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_NonStandardUpdateRequest.DiscardUnknown(m)
}

var xxx_messageInfo_NonStandardUpdateRequest proto.InternalMessageInfo

func (m *NonStandardUpdateRequest) GetBody() *NonStandardMessage {
	if m != nil {
		return m.Body
	}
	return nil
}

func (m *NonStandardUpdateRequest) GetUpdateMask() *field_mask.FieldMask {
	if m != nil {
		return m.UpdateMask
	}
	return nil
}

// NonStandardMessageWithJSONNames maps odd field names to odd JSON names for maximum confusion.
type NonStandardMessageWithJSONNames struct {
	// Id represents the message identifier.
	Id                   string                                 `protobuf:"bytes,1,opt,name=id,json=ID,proto3" json:"id,omitempty"`
	Num                  int64                                  `protobuf:"varint,2,opt,name=Num,proto3" json:"Num,omitempty"`
	LineNum              int64                                  `protobuf:"varint,3,opt,name=line_num,json=LineNum,proto3" json:"line_num,omitempty"`
	LangIdent            string                                 `protobuf:"bytes,4,opt,name=langIdent,proto3" json:"langIdent,omitempty"`
	STATUS               string                                 `protobuf:"bytes,5,opt,name=STATUS,json=status,proto3" json:"STATUS,omitempty"`
	En_GB                int64                                  `protobuf:"varint,6,opt,name=en_GB,json=En_GB,proto3" json:"en_GB,omitempty"`
	No                   string                                 `protobuf:"bytes,7,opt,name=no,json=yes,proto3" json:"no,omitempty"`
	Thing                *NonStandardMessageWithJSONNames_Thing `protobuf:"bytes,8,opt,name=thing,json=Thingy,proto3" json:"thing,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                               `json:"-"`
	XXX_unrecognized     []byte                                 `json:"-"`
	XXX_sizecache        int32                                  `json:"-"`
}

func (m *NonStandardMessageWithJSONNames) Reset()         { *m = NonStandardMessageWithJSONNames{} }
func (m *NonStandardMessageWithJSONNames) String() string { return proto.CompactTextString(m) }
func (*NonStandardMessageWithJSONNames) ProtoMessage()    {}
func (*NonStandardMessageWithJSONNames) Descriptor() ([]byte, []int) {
	return fileDescriptor_a1fc3d96d9306f6b, []int{2}
}

func (m *NonStandardMessageWithJSONNames) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_NonStandardMessageWithJSONNames.Unmarshal(m, b)
}
func (m *NonStandardMessageWithJSONNames) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_NonStandardMessageWithJSONNames.Marshal(b, m, deterministic)
}
func (m *NonStandardMessageWithJSONNames) XXX_Merge(src proto.Message) {
	xxx_messageInfo_NonStandardMessageWithJSONNames.Merge(m, src)
}
func (m *NonStandardMessageWithJSONNames) XXX_Size() int {
	return xxx_messageInfo_NonStandardMessageWithJSONNames.Size(m)
}
func (m *NonStandardMessageWithJSONNames) XXX_DiscardUnknown() {
	xxx_messageInfo_NonStandardMessageWithJSONNames.DiscardUnknown(m)
}

var xxx_messageInfo_NonStandardMessageWithJSONNames proto.InternalMessageInfo

func (m *NonStandardMessageWithJSONNames) GetId() string {
	if m != nil {
		return m.Id
	}
	return ""
}

func (m *NonStandardMessageWithJSONNames) GetNum() int64 {
	if m != nil {
		return m.Num
	}
	return 0
}

func (m *NonStandardMessageWithJSONNames) GetLineNum() int64 {
	if m != nil {
		return m.LineNum
	}
	return 0
}

func (m *NonStandardMessageWithJSONNames) GetLangIdent() string {
	if m != nil {
		return m.LangIdent
	}
	return ""
}

func (m *NonStandardMessageWithJSONNames) GetSTATUS() string {
	if m != nil {
		return m.STATUS
	}
	return ""
}

func (m *NonStandardMessageWithJSONNames) GetEn_GB() int64 {
	if m != nil {
		return m.En_GB
	}
	return 0
}

func (m *NonStandardMessageWithJSONNames) GetNo() string {
	if m != nil {
		return m.No
	}
	return ""
}

func (m *NonStandardMessageWithJSONNames) GetThing() *NonStandardMessageWithJSONNames_Thing {
	if m != nil {
		return m.Thing
	}
	return nil
}

type NonStandardMessageWithJSONNames_Thing struct {
	SubThing             *NonStandardMessageWithJSONNames_Thing_SubThing `protobuf:"bytes,1,opt,name=subThing,json=SubThing,proto3" json:"subThing,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                                        `json:"-"`
	XXX_unrecognized     []byte                                          `json:"-"`
	XXX_sizecache        int32                                           `json:"-"`
}

func (m *NonStandardMessageWithJSONNames_Thing) Reset()         { *m = NonStandardMessageWithJSONNames_Thing{} }
func (m *NonStandardMessageWithJSONNames_Thing) String() string { return proto.CompactTextString(m) }
func (*NonStandardMessageWithJSONNames_Thing) ProtoMessage()    {}
func (*NonStandardMessageWithJSONNames_Thing) Descriptor() ([]byte, []int) {
	return fileDescriptor_a1fc3d96d9306f6b, []int{2, 0}
}

func (m *NonStandardMessageWithJSONNames_Thing) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_NonStandardMessageWithJSONNames_Thing.Unmarshal(m, b)
}
func (m *NonStandardMessageWithJSONNames_Thing) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_NonStandardMessageWithJSONNames_Thing.Marshal(b, m, deterministic)
}
func (m *NonStandardMessageWithJSONNames_Thing) XXX_Merge(src proto.Message) {
	xxx_messageInfo_NonStandardMessageWithJSONNames_Thing.Merge(m, src)
}
func (m *NonStandardMessageWithJSONNames_Thing) XXX_Size() int {
	return xxx_messageInfo_NonStandardMessageWithJSONNames_Thing.Size(m)
}
func (m *NonStandardMessageWithJSONNames_Thing) XXX_DiscardUnknown() {
	xxx_messageInfo_NonStandardMessageWithJSONNames_Thing.DiscardUnknown(m)
}

var xxx_messageInfo_NonStandardMessageWithJSONNames_Thing proto.InternalMessageInfo

func (m *NonStandardMessageWithJSONNames_Thing) GetSubThing() *NonStandardMessageWithJSONNames_Thing_SubThing {
	if m != nil {
		return m.SubThing
	}
	return nil
}

type NonStandardMessageWithJSONNames_Thing_SubThing struct {
	SubValue             string   `protobuf:"bytes,1,opt,name=sub_value,json=sub_Value,proto3" json:"sub_value,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *NonStandardMessageWithJSONNames_Thing_SubThing) Reset() {
	*m = NonStandardMessageWithJSONNames_Thing_SubThing{}
}
func (m *NonStandardMessageWithJSONNames_Thing_SubThing) String() string {
	return proto.CompactTextString(m)
}
func (*NonStandardMessageWithJSONNames_Thing_SubThing) ProtoMessage() {}
func (*NonStandardMessageWithJSONNames_Thing_SubThing) Descriptor() ([]byte, []int) {
	return fileDescriptor_a1fc3d96d9306f6b, []int{2, 0, 0}
}

func (m *NonStandardMessageWithJSONNames_Thing_SubThing) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_NonStandardMessageWithJSONNames_Thing_SubThing.Unmarshal(m, b)
}
func (m *NonStandardMessageWithJSONNames_Thing_SubThing) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_NonStandardMessageWithJSONNames_Thing_SubThing.Marshal(b, m, deterministic)
}
func (m *NonStandardMessageWithJSONNames_Thing_SubThing) XXX_Merge(src proto.Message) {
	xxx_messageInfo_NonStandardMessageWithJSONNames_Thing_SubThing.Merge(m, src)
}
func (m *NonStandardMessageWithJSONNames_Thing_SubThing) XXX_Size() int {
	return xxx_messageInfo_NonStandardMessageWithJSONNames_Thing_SubThing.Size(m)
}
func (m *NonStandardMessageWithJSONNames_Thing_SubThing) XXX_DiscardUnknown() {
	xxx_messageInfo_NonStandardMessageWithJSONNames_Thing_SubThing.DiscardUnknown(m)
}

var xxx_messageInfo_NonStandardMessageWithJSONNames_Thing_SubThing proto.InternalMessageInfo

func (m *NonStandardMessageWithJSONNames_Thing_SubThing) GetSubValue() string {
	if m != nil {
		return m.SubValue
	}
	return ""
}

type NonStandardWithJSONNamesUpdateRequest struct {
	Body                 *NonStandardMessageWithJSONNames `protobuf:"bytes,1,opt,name=body,proto3" json:"body,omitempty"`
	UpdateMask           *field_mask.FieldMask            `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                         `json:"-"`
	XXX_unrecognized     []byte                           `json:"-"`
	XXX_sizecache        int32                            `json:"-"`
}

func (m *NonStandardWithJSONNamesUpdateRequest) Reset()         { *m = NonStandardWithJSONNamesUpdateRequest{} }
func (m *NonStandardWithJSONNamesUpdateRequest) String() string { return proto.CompactTextString(m) }
func (*NonStandardWithJSONNamesUpdateRequest) ProtoMessage()    {}
func (*NonStandardWithJSONNamesUpdateRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_a1fc3d96d9306f6b, []int{3}
}

func (m *NonStandardWithJSONNamesUpdateRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_NonStandardWithJSONNamesUpdateRequest.Unmarshal(m, b)
}
func (m *NonStandardWithJSONNamesUpdateRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_NonStandardWithJSONNamesUpdateRequest.Marshal(b, m, deterministic)
}
func (m *NonStandardWithJSONNamesUpdateRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_NonStandardWithJSONNamesUpdateRequest.Merge(m, src)
}
func (m *NonStandardWithJSONNamesUpdateRequest) XXX_Size() int {
	return xxx_messageInfo_NonStandardWithJSONNamesUpdateRequest.Size(m)
}
func (m *NonStandardWithJSONNamesUpdateRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_NonStandardWithJSONNamesUpdateRequest.DiscardUnknown(m)
}

var xxx_messageInfo_NonStandardWithJSONNamesUpdateRequest proto.InternalMessageInfo

func (m *NonStandardWithJSONNamesUpdateRequest) GetBody() *NonStandardMessageWithJSONNames {
	if m != nil {
		return m.Body
	}
	return nil
}

func (m *NonStandardWithJSONNamesUpdateRequest) GetUpdateMask() *field_mask.FieldMask {
	if m != nil {
		return m.UpdateMask
	}
	return nil
}

func init() {
	proto.RegisterType((*NonStandardMessage)(nil), "grpc.gateway.examples.internal.examplepb.NonStandardMessage")
	proto.RegisterType((*NonStandardMessage_Thing)(nil), "grpc.gateway.examples.internal.examplepb.NonStandardMessage.Thing")
	proto.RegisterType((*NonStandardMessage_Thing_SubThing)(nil), "grpc.gateway.examples.internal.examplepb.NonStandardMessage.Thing.SubThing")
	proto.RegisterType((*NonStandardUpdateRequest)(nil), "grpc.gateway.examples.internal.examplepb.NonStandardUpdateRequest")
	proto.RegisterType((*NonStandardMessageWithJSONNames)(nil), "grpc.gateway.examples.internal.examplepb.NonStandardMessageWithJSONNames")
	proto.RegisterType((*NonStandardMessageWithJSONNames_Thing)(nil), "grpc.gateway.examples.internal.examplepb.NonStandardMessageWithJSONNames.Thing")
	proto.RegisterType((*NonStandardMessageWithJSONNames_Thing_SubThing)(nil), "grpc.gateway.examples.internal.examplepb.NonStandardMessageWithJSONNames.Thing.SubThing")
	proto.RegisterType((*NonStandardWithJSONNamesUpdateRequest)(nil), "grpc.gateway.examples.internal.examplepb.NonStandardWithJSONNamesUpdateRequest")
}

func init() {
	proto.RegisterFile("examples/internal/proto/examplepb/non_standard_names.proto", fileDescriptor_a1fc3d96d9306f6b)
}

var fileDescriptor_a1fc3d96d9306f6b = []byte{
	// 636 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xac, 0x95, 0x4f, 0x6f, 0xd3, 0x3e,
	0x18, 0xc7, 0x95, 0x66, 0xed, 0x5a, 0x57, 0xfa, 0xfd, 0x90, 0x87, 0x50, 0x08, 0x93, 0x56, 0x55,
	0x42, 0xeb, 0x05, 0x47, 0x84, 0x03, 0xd2, 0x80, 0x03, 0x15, 0x30, 0x0d, 0x58, 0x87, 0x92, 0x0e,
	0x26, 0x24, 0x14, 0x39, 0x8b, 0x9b, 0x86, 0xa5, 0x76, 0xa8, 0x9d, 0x8e, 0x5e, 0x79, 0x0b, 0x9c,
	0xb8, 0x72, 0x41, 0xbc, 0x04, 0xce, 0x3b, 0x73, 0xe2, 0x2d, 0xf0, 0x02, 0x78, 0x09, 0x28, 0x76,
	0x52, 0x1a, 0x55, 0x88, 0xa9, 0xed, 0x29, 0x7e, 0x1e, 0x5b, 0xdf, 0xe7, 0xdf, 0x27, 0x36, 0xd8,
	0x23, 0xef, 0xf1, 0x28, 0x89, 0x09, 0xb7, 0x22, 0x2a, 0xc8, 0x98, 0xe2, 0xd8, 0x4a, 0xc6, 0x4c,
	0x30, 0x2b, 0xf7, 0x27, 0xbe, 0x45, 0x19, 0xf5, 0xb8, 0xc0, 0x34, 0xc0, 0xe3, 0xc0, 0xa3, 0x78,
	0x44, 0x38, 0x92, 0x47, 0x60, 0x27, 0x1c, 0x27, 0xa7, 0x28, 0xc4, 0x82, 0x9c, 0xe3, 0x29, 0x2a,
	0x84, 0x50, 0x21, 0x84, 0x66, 0x12, 0xe6, 0x76, 0xc8, 0x58, 0x18, 0x13, 0x0b, 0x27, 0x91, 0x85,
	0x29, 0x65, 0x02, 0x8b, 0x88, 0xd1, 0x5c, 0xc7, 0x6c, 0xe5, 0xbb, 0xd2, 0xf2, 0xd3, 0x81, 0x35,
	0x88, 0x48, 0x1c, 0x78, 0x23, 0xcc, 0xcf, 0xd4, 0x89, 0xf6, 0x67, 0x1d, 0xc0, 0x1e, 0xa3, 0x6e,
	0x9e, 0xc5, 0x21, 0xe1, 0x1c, 0x87, 0x04, 0xfe, 0x07, 0x2a, 0x51, 0x60, 0x68, 0x2d, 0xad, 0xd3,
	0x70, 0x2a, 0x51, 0x00, 0xaf, 0x00, 0xbd, 0x97, 0x8e, 0x8c, 0x4a, 0x4b, 0xeb, 0xe8, 0x4e, 0xb6,
	0x84, 0xd7, 0x41, 0x3d, 0x8e, 0x28, 0xf1, 0x68, 0x3a, 0x32, 0x74, 0xe9, 0xde, 0xcc, 0xec, 0x6c,
	0x6b, 0x1b, 0x34, 0x62, 0x4c, 0xc3, 0x83, 0x80, 0x50, 0x61, 0x6c, 0x48, 0x8d, 0x3f, 0x0e, 0x78,
	0x0d, 0xd4, 0xdc, 0xfe, 0xc3, 0xfe, 0xb1, 0x6b, 0x54, 0xe5, 0x56, 0x6e, 0xc1, 0x2d, 0x50, 0x25,
	0xd4, 0xdb, 0xef, 0x1a, 0x35, 0xa9, 0xb6, 0x41, 0xe8, 0x7e, 0x37, 0xcb, 0x83, 0x32, 0x63, 0x53,
	0xe5, 0x41, 0x19, 0x3c, 0x01, 0x55, 0x31, 0x8c, 0x68, 0x68, 0xd4, 0x5b, 0x5a, 0xa7, 0x69, 0x77,
	0xd1, 0x65, 0x1b, 0x85, 0x16, 0x8b, 0x44, 0xfd, 0x4c, 0xc9, 0x51, 0x82, 0xe6, 0x27, 0x0d, 0x54,
	0xa5, 0x03, 0x86, 0xa0, 0xce, 0x53, 0x5f, 0xae, 0x65, 0x07, 0x9a, 0xf6, 0xb3, 0xd5, 0xc3, 0x20,
	0x37, 0x97, 0x74, 0x66, 0xe2, 0xe6, 0x2e, 0xa8, 0x17, 0x5e, 0x78, 0x03, 0x34, 0x78, 0xea, 0x7b,
	0x13, 0x1c, 0xa7, 0x24, 0xef, 0x7b, 0x76, 0xf0, 0x65, 0x66, 0xb7, 0xbf, 0x6a, 0xc0, 0x98, 0x13,
	0x3e, 0x4e, 0x02, 0x2c, 0x88, 0x43, 0xde, 0xa5, 0x84, 0x0b, 0xf8, 0x02, 0x6c, 0xf8, 0x2c, 0x98,
	0xe6, 0xa9, 0xde, 0x5f, 0x25, 0x55, 0x47, 0x2a, 0xc1, 0x7b, 0xa0, 0x99, 0xca, 0x10, 0x12, 0x14,
	0x39, 0xf4, 0xa6, 0x6d, 0x22, 0xc5, 0x12, 0x2a, 0x58, 0x42, 0x4f, 0x32, 0x96, 0x0e, 0x31, 0x3f,
	0x73, 0x80, 0x3a, 0x9e, 0xad, 0xdb, 0xdf, 0x75, 0xb0, 0xb3, 0xa8, 0xfc, 0x2a, 0x12, 0xc3, 0xa7,
	0xee, 0x51, 0xaf, 0x97, 0x41, 0x5e, 0xa6, 0xeb, 0xe0, 0xd1, 0xe5, 0xe8, 0x7a, 0xbe, 0x1c, 0x5d,
	0x5c, 0x60, 0x91, 0x72, 0x78, 0xb5, 0x4c, 0x57, 0xf5, 0x71, 0x66, 0xc0, 0xff, 0xe7, 0xf0, 0xd2,
	0xa7, 0x84, 0xc3, 0x41, 0x99, 0xaf, 0xa3, 0x55, 0xba, 0x59, 0xaa, 0x39, 0x87, 0xad, 0x26, 0x3f,
	0x53, 0xf3, 0xcb, 0x8c, 0x36, 0xb1, 0x40, 0xdb, 0xc9, 0x9a, 0x83, 0xce, 0xa1, 0x57, 0xac, 0xcc,
	0xce, 0x1c, 0x7a, 0xdb, 0x8b, 0xe8, 0x49, 0x87, 0x62, 0xef, 0x42, 0x03, 0x37, 0xe7, 0xc2, 0x94,
	0xf4, 0xcb, 0x20, 0xbe, 0x29, 0x81, 0x78, 0xb0, 0xb6, 0x2a, 0xd6, 0x40, 0xa5, 0x7d, 0x51, 0xbe,
	0xe6, 0x5c, 0x32, 0x9e, 0x44, 0xa7, 0x04, 0x7e, 0xd3, 0x40, 0x4d, 0x15, 0x01, 0x97, 0xbb, 0x4a,
	0x4a, 0x1d, 0x30, 0x57, 0xfa, 0xf9, 0xda, 0xb7, 0x3e, 0xfc, 0xf8, 0xf9, 0xb1, 0xb2, 0x6b, 0xef,
	0x58, 0x93, 0xdb, 0xc5, 0x23, 0x51, 0x7a, 0x22, 0x2c, 0x55, 0xd0, 0x9e, 0xea, 0xc7, 0x2f, 0x0d,
	0x6c, 0xa9, 0xf0, 0xe5, 0x9f, 0x6b, 0x39, 0x66, 0xff, 0x3e, 0x57, 0x73, 0x7d, 0x93, 0x6c, 0x3f,
	0x90, 0x25, 0xde, 0xb5, 0xad, 0x7f, 0x94, 0xe8, 0x9d, 0x47, 0x62, 0xe8, 0xbd, 0xe5, 0x8c, 0xaa,
	0x67, 0x51, 0x95, 0xdc, 0x6d, 0xbe, 0x6e, 0xcc, 0x62, 0xf9, 0x35, 0x39, 0xf2, 0x3b, 0xbf, 0x03,
	0x00, 0x00, 0xff, 0xff, 0x0f, 0xef, 0xfa, 0x6c, 0x68, 0x07, 0x00, 0x00,
}

// Reference imports to suppress errors if they are not otherwise used.
var _ context.Context
var _ grpc.ClientConnInterface

// This is a compile-time assertion to ensure that this generated file
// is compatible with the grpc package it is being compiled against.
const _ = grpc.SupportPackageIsVersion6

// NonStandardServiceClient is the client API for NonStandardService service.
//
// For semantics around ctx use and closing/ending streaming RPCs, please refer to https://godoc.org/google.golang.org/grpc#ClientConn.NewStream.
type NonStandardServiceClient interface {
	// Apply field mask to empty NonStandardMessage and return result.
	Update(ctx context.Context, in *NonStandardUpdateRequest, opts ...grpc.CallOption) (*NonStandardMessage, error)
	// Apply field mask to empty NonStandardMessageWithJSONNames and return result.
	UpdateWithJSONNames(ctx context.Context, in *NonStandardWithJSONNamesUpdateRequest, opts ...grpc.CallOption) (*NonStandardMessageWithJSONNames, error)
}

type nonStandardServiceClient struct {
	cc grpc.ClientConnInterface
}

func NewNonStandardServiceClient(cc grpc.ClientConnInterface) NonStandardServiceClient {
	return &nonStandardServiceClient{cc}
}

func (c *nonStandardServiceClient) Update(ctx context.Context, in *NonStandardUpdateRequest, opts ...grpc.CallOption) (*NonStandardMessage, error) {
	out := new(NonStandardMessage)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.NonStandardService/Update", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *nonStandardServiceClient) UpdateWithJSONNames(ctx context.Context, in *NonStandardWithJSONNamesUpdateRequest, opts ...grpc.CallOption) (*NonStandardMessageWithJSONNames, error) {
	out := new(NonStandardMessageWithJSONNames)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.NonStandardService/UpdateWithJSONNames", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

// NonStandardServiceServer is the server API for NonStandardService service.
type NonStandardServiceServer interface {
	// Apply field mask to empty NonStandardMessage and return result.
	Update(context.Context, *NonStandardUpdateRequest) (*NonStandardMessage, error)
	// Apply field mask to empty NonStandardMessageWithJSONNames and return result.
	UpdateWithJSONNames(context.Context, *NonStandardWithJSONNamesUpdateRequest) (*NonStandardMessageWithJSONNames, error)
}

// UnimplementedNonStandardServiceServer can be embedded to have forward compatible implementations.
type UnimplementedNonStandardServiceServer struct {
}

func (*UnimplementedNonStandardServiceServer) Update(ctx context.Context, req *NonStandardUpdateRequest) (*NonStandardMessage, error) {
	return nil, status.Errorf(codes.Unimplemented, "method Update not implemented")
}
func (*UnimplementedNonStandardServiceServer) UpdateWithJSONNames(ctx context.Context, req *NonStandardWithJSONNamesUpdateRequest) (*NonStandardMessageWithJSONNames, error) {
	return nil, status.Errorf(codes.Unimplemented, "method UpdateWithJSONNames not implemented")
}

func RegisterNonStandardServiceServer(s *grpc.Server, srv NonStandardServiceServer) {
	s.RegisterService(&_NonStandardService_serviceDesc, srv)
}

func _NonStandardService_Update_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(NonStandardUpdateRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(NonStandardServiceServer).Update(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.NonStandardService/Update",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(NonStandardServiceServer).Update(ctx, req.(*NonStandardUpdateRequest))
	}
	return interceptor(ctx, in, info, handler)
}

func _NonStandardService_UpdateWithJSONNames_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(NonStandardWithJSONNamesUpdateRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(NonStandardServiceServer).UpdateWithJSONNames(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.NonStandardService/UpdateWithJSONNames",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(NonStandardServiceServer).UpdateWithJSONNames(ctx, req.(*NonStandardWithJSONNamesUpdateRequest))
	}
	return interceptor(ctx, in, info, handler)
}

var _NonStandardService_serviceDesc = grpc.ServiceDesc{
	ServiceName: "grpc.gateway.examples.internal.examplepb.NonStandardService",
	HandlerType: (*NonStandardServiceServer)(nil),
	Methods: []grpc.MethodDesc{
		{
			MethodName: "Update",
			Handler:    _NonStandardService_Update_Handler,
		},
		{
			MethodName: "UpdateWithJSONNames",
			Handler:    _NonStandardService_UpdateWithJSONNames_Handler,
		},
	},
	Streams:  []grpc.StreamDesc{},
	Metadata: "examples/internal/proto/examplepb/non_standard_names.proto",
}
