// Code generated by protoc-gen-go. DO NOT EDIT.
// source: examples/internal/proto/examplepb/wrappers.proto

package examplepb

import (
	context "context"
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	empty "github.com/golang/protobuf/ptypes/empty"
	wrappers "github.com/golang/protobuf/ptypes/wrappers"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	grpc "google.golang.org/grpc"
	codes "google.golang.org/grpc/codes"
	status "google.golang.org/grpc/status"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type Wrappers struct {
	StringValue          *wrappers.StringValue `protobuf:"bytes,1,opt,name=string_value,json=stringValue,proto3" json:"string_value,omitempty"`
	Int32Value           *wrappers.Int32Value  `protobuf:"bytes,2,opt,name=int32_value,json=int32Value,proto3" json:"int32_value,omitempty"`
	Int64Value           *wrappers.Int64Value  `protobuf:"bytes,3,opt,name=int64_value,json=int64Value,proto3" json:"int64_value,omitempty"`
	FloatValue           *wrappers.FloatValue  `protobuf:"bytes,4,opt,name=float_value,json=floatValue,proto3" json:"float_value,omitempty"`
	DoubleValue          *wrappers.DoubleValue `protobuf:"bytes,5,opt,name=double_value,json=doubleValue,proto3" json:"double_value,omitempty"`
	BoolValue            *wrappers.BoolValue   `protobuf:"bytes,6,opt,name=bool_value,json=boolValue,proto3" json:"bool_value,omitempty"`
	Uint32Value          *wrappers.UInt32Value `protobuf:"bytes,7,opt,name=uint32_value,json=uint32Value,proto3" json:"uint32_value,omitempty"`
	Uint64Value          *wrappers.UInt64Value `protobuf:"bytes,8,opt,name=uint64_value,json=uint64Value,proto3" json:"uint64_value,omitempty"`
	BytesValue           *wrappers.BytesValue  `protobuf:"bytes,9,opt,name=bytes_value,json=bytesValue,proto3" json:"bytes_value,omitempty"`
	XXX_NoUnkeyedLiteral struct{}              `json:"-"`
	XXX_unrecognized     []byte                `json:"-"`
	XXX_sizecache        int32                 `json:"-"`
}

func (m *Wrappers) Reset()         { *m = Wrappers{} }
func (m *Wrappers) String() string { return proto.CompactTextString(m) }
func (*Wrappers) ProtoMessage()    {}
func (*Wrappers) Descriptor() ([]byte, []int) {
	return fileDescriptor_70304970d74748a4, []int{0}
}

func (m *Wrappers) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_Wrappers.Unmarshal(m, b)
}
func (m *Wrappers) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_Wrappers.Marshal(b, m, deterministic)
}
func (m *Wrappers) XXX_Merge(src proto.Message) {
	xxx_messageInfo_Wrappers.Merge(m, src)
}
func (m *Wrappers) XXX_Size() int {
	return xxx_messageInfo_Wrappers.Size(m)
}
func (m *Wrappers) XXX_DiscardUnknown() {
	xxx_messageInfo_Wrappers.DiscardUnknown(m)
}

var xxx_messageInfo_Wrappers proto.InternalMessageInfo

func (m *Wrappers) GetStringValue() *wrappers.StringValue {
	if m != nil {
		return m.StringValue
	}
	return nil
}

func (m *Wrappers) GetInt32Value() *wrappers.Int32Value {
	if m != nil {
		return m.Int32Value
	}
	return nil
}

func (m *Wrappers) GetInt64Value() *wrappers.Int64Value {
	if m != nil {
		return m.Int64Value
	}
	return nil
}

func (m *Wrappers) GetFloatValue() *wrappers.FloatValue {
	if m != nil {
		return m.FloatValue
	}
	return nil
}

func (m *Wrappers) GetDoubleValue() *wrappers.DoubleValue {
	if m != nil {
		return m.DoubleValue
	}
	return nil
}

func (m *Wrappers) GetBoolValue() *wrappers.BoolValue {
	if m != nil {
		return m.BoolValue
	}
	return nil
}

func (m *Wrappers) GetUint32Value() *wrappers.UInt32Value {
	if m != nil {
		return m.Uint32Value
	}
	return nil
}

func (m *Wrappers) GetUint64Value() *wrappers.UInt64Value {
	if m != nil {
		return m.Uint64Value
	}
	return nil
}

func (m *Wrappers) GetBytesValue() *wrappers.BytesValue {
	if m != nil {
		return m.BytesValue
	}
	return nil
}

func init() {
	proto.RegisterType((*Wrappers)(nil), "grpc.gateway.examples.internal.examplepb.Wrappers")
}

func init() {
	proto.RegisterFile("examples/internal/proto/examplepb/wrappers.proto", fileDescriptor_70304970d74748a4)
}

var fileDescriptor_70304970d74748a4 = []byte{
	// 587 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x9c, 0x95, 0xdd, 0x6e, 0xd3, 0x30,
	0x14, 0xc7, 0xd5, 0x31, 0xca, 0xea, 0x0c, 0xc6, 0x0c, 0x82, 0xcd, 0x9b, 0x00, 0xe5, 0x6a, 0xda,
	0x85, 0x03, 0x5d, 0x55, 0x89, 0x09, 0x09, 0xa9, 0x7c, 0x48, 0xdc, 0x32, 0x8d, 0x49, 0xdc, 0x20,
	0x67, 0x73, 0xa3, 0xa0, 0x2c, 0x8e, 0x12, 0xa7, 0xa3, 0xb7, 0xbc, 0x01, 0xe2, 0x4d, 0x78, 0x03,
	0x9e, 0x81, 0x57, 0xe0, 0x41, 0x90, 0xbf, 0xf2, 0xd1, 0x2c, 0x47, 0x68, 0x77, 0x8e, 0xff, 0xe7,
	0x77, 0xfe, 0xf5, 0xf1, 0x39, 0x2e, 0x7a, 0xce, 0xbf, 0xb1, 0xcb, 0x2c, 0xe1, 0x45, 0x10, 0xa7,
	0x92, 0xe7, 0x29, 0x4b, 0x82, 0x2c, 0x17, 0x52, 0x04, 0x76, 0x3f, 0x0b, 0x83, 0xab, 0x9c, 0x65,
	0x19, 0xcf, 0x0b, 0xaa, 0x05, 0x7c, 0x10, 0xe5, 0xd9, 0x39, 0x8d, 0x98, 0xe4, 0x57, 0x6c, 0x49,
	0x1d, 0x4e, 0x1d, 0x4e, 0x2b, 0x90, 0xec, 0x47, 0x42, 0x44, 0x09, 0x0f, 0x58, 0x16, 0x07, 0x2c,
	0x4d, 0x85, 0x64, 0x32, 0x16, 0xa9, 0xcd, 0x43, 0x9e, 0x58, 0x55, 0x7f, 0x85, 0xe5, 0x7c, 0xc5,
	0x87, 0xec, 0xad, 0xea, 0xfc, 0x32, 0x93, 0x4b, 0x23, 0xfa, 0xbf, 0xd7, 0xd1, 0xc6, 0x99, 0x8d,
	0xc7, 0xaf, 0xd1, 0x66, 0x21, 0xf3, 0x38, 0x8d, 0xbe, 0x2c, 0x58, 0x52, 0xf2, 0x9d, 0xc1, 0xb3,
	0xc1, 0x81, 0x37, 0xde, 0xa7, 0x26, 0x01, 0x75, 0x09, 0xe8, 0x89, 0x0e, 0xfa, 0xa4, 0x62, 0x3e,
	0x7a, 0x45, 0xfd, 0x81, 0x5f, 0x21, 0x2f, 0x4e, 0xe5, 0xd1, 0xd8, 0xf2, 0x6b, 0x9a, 0xdf, 0xeb,
	0xf0, 0x1f, 0x54, 0x8c, 0xc1, 0x51, 0x5c, 0xad, 0x2d, 0x3d, 0x9d, 0x58, 0xfa, 0x56, 0x3f, 0x3d,
	0x9d, 0xd4, 0xb4, 0x5d, 0x2b, 0x7a, 0x9e, 0x08, 0x26, 0x2d, 0xbd, 0xde, 0x43, 0xbf, 0x57, 0x31,
	0x96, 0x9e, 0x57, 0x6b, 0x75, 0xf4, 0x0b, 0x51, 0x86, 0x09, 0xb7, 0xf8, 0xed, 0x9e, 0xa3, 0xbf,
	0xd5, 0x41, 0xf6, 0xe8, 0x17, 0xf5, 0x07, 0x7e, 0x89, 0x50, 0x28, 0x44, 0x62, 0xf1, 0xa1, 0xc6,
	0x49, 0x07, 0x9f, 0x09, 0x91, 0x18, 0x78, 0x14, 0xba, 0xa5, 0xf2, 0x2e, 0x9b, 0x65, 0xbb, 0xd3,
	0xe3, 0x7d, 0xda, 0xa8, 0x9b, 0x57, 0x36, 0x0a, 0x67, 0x13, 0x54, 0x95, 0xdb, 0x00, 0x12, 0xb8,
	0xd2, 0x79, 0x65, 0xbb, 0x76, 0xe1, 0x52, 0xf2, 0xc2, 0xf2, 0xa3, 0x9e, 0xda, 0xcd, 0x54, 0x8c,
	0xad, 0x5d, 0x58, 0xad, 0xc7, 0xbf, 0x46, 0x68, 0xcb, 0xf5, 0xd0, 0x09, 0xcf, 0x17, 0xf1, 0x39,
	0xc7, 0x3f, 0x06, 0x68, 0xf8, 0x26, 0xe7, 0x4c, 0x72, 0x3c, 0xa6, 0xff, 0xdb, 0xe8, 0xd4, 0x65,
	0x21, 0x37, 0x60, 0xfc, 0xa7, 0xdf, 0xff, 0xfc, 0xfd, 0xb9, 0xb6, 0xeb, 0x3f, 0x0c, 0x16, 0x2f,
	0xdc, 0xd4, 0x55, 0xb3, 0x70, 0x3c, 0x38, 0xc4, 0x5f, 0xd1, 0xb6, 0xf9, 0x49, 0x8d, 0xfe, 0xc5,
	0x60, 0x77, 0x13, 0x50, 0xf5, 0x77, 0xb5, 0xe3, 0x03, 0xff, 0x9e, 0x72, 0x94, 0xbc, 0x90, 0x46,
	0x54, 0x5e, 0x73, 0x74, 0xdf, 0x78, 0xd5, 0x77, 0x86, 0xa1, 0x41, 0x20, 0x90, 0xe8, 0xef, 0x68,
	0x23, 0xec, 0xdf, 0x75, 0x46, 0x5a, 0x5b, 0xf5, 0x71, 0xb7, 0x09, 0x8d, 0x0c, 0x81, 0xc4, 0x6b,
	0x7d, 0xa6, 0x93, 0x96, 0x4f, 0x3d, 0x3f, 0x18, 0x1a, 0x2e, 0x02, 0x89, 0x5d, 0x1f, 0xad, 0xb5,
	0xee, 0xa8, 0x31, 0x68, 0x18, 0x1c, 0x43, 0x02, 0xaa, 0xdd, 0x3b, 0x32, 0xa2, 0xf2, 0x0a, 0xd1,
	0x96, 0xf1, 0xaa, 0xa6, 0x12, 0x03, 0x13, 0x4b, 0x00, 0xcd, 0x7f, 0xac, 0x5d, 0xb6, 0xfd, 0x4d,
	0xe7, 0xa2, 0xa4, 0xd6, 0x79, 0x1a, 0xc3, 0x8b, 0xc1, 0xd1, 0x26, 0xa0, 0xda, 0x3d, 0xcf, 0x69,
	0xec, 0x7a, 0xa1, 0xe5, 0xe5, 0x9a, 0x01, 0x7c, 0x05, 0x08, 0xa8, 0x5e, 0xef, 0xb5, 0xd2, 0x0f,
	0xf5, 0x9b, 0x80, 0xa1, 0x07, 0x83, 0x40, 0x62, 0xb7, 0x1f, 0xb4, 0xa6, 0x7c, 0xce, 0x90, 0x67,
	0x7c, 0xde, 0xa9, 0x3f, 0x2d, 0xfc, 0xa8, 0x93, 0x45, 0xef, 0x93, 0x9e, 0xfd, 0x6e, 0x62, 0xbd,
	0x7d, 0x3c, 0x38, 0x9c, 0x79, 0x9f, 0x47, 0xd5, 0x1b, 0x12, 0x0e, 0x35, 0x76, 0xf4, 0x2f, 0x00,
	0x00, 0xff, 0xff, 0xfe, 0xb0, 0x38, 0x00, 0xc5, 0x07, 0x00, 0x00,
}

// Reference imports to suppress errors if they are not otherwise used.
var _ context.Context
var _ grpc.ClientConnInterface

// This is a compile-time assertion to ensure that this generated file
// is compatible with the grpc package it is being compiled against.
const _ = grpc.SupportPackageIsVersion6

// WrappersServiceClient is the client API for WrappersService service.
//
// For semantics around ctx use and closing/ending streaming RPCs, please refer to https://godoc.org/google.golang.org/grpc#ClientConn.NewStream.
type WrappersServiceClient interface {
	Create(ctx context.Context, in *Wrappers, opts ...grpc.CallOption) (*Wrappers, error)
	CreateStringValue(ctx context.Context, in *wrappers.StringValue, opts ...grpc.CallOption) (*wrappers.StringValue, error)
	CreateInt32Value(ctx context.Context, in *wrappers.Int32Value, opts ...grpc.CallOption) (*wrappers.Int32Value, error)
	CreateInt64Value(ctx context.Context, in *wrappers.Int64Value, opts ...grpc.CallOption) (*wrappers.Int64Value, error)
	CreateFloatValue(ctx context.Context, in *wrappers.FloatValue, opts ...grpc.CallOption) (*wrappers.FloatValue, error)
	CreateDoubleValue(ctx context.Context, in *wrappers.DoubleValue, opts ...grpc.CallOption) (*wrappers.DoubleValue, error)
	CreateBoolValue(ctx context.Context, in *wrappers.BoolValue, opts ...grpc.CallOption) (*wrappers.BoolValue, error)
	CreateUInt32Value(ctx context.Context, in *wrappers.UInt32Value, opts ...grpc.CallOption) (*wrappers.UInt32Value, error)
	CreateUInt64Value(ctx context.Context, in *wrappers.UInt64Value, opts ...grpc.CallOption) (*wrappers.UInt64Value, error)
	CreateBytesValue(ctx context.Context, in *wrappers.BytesValue, opts ...grpc.CallOption) (*wrappers.BytesValue, error)
	CreateEmpty(ctx context.Context, in *empty.Empty, opts ...grpc.CallOption) (*empty.Empty, error)
}

type wrappersServiceClient struct {
	cc grpc.ClientConnInterface
}

func NewWrappersServiceClient(cc grpc.ClientConnInterface) WrappersServiceClient {
	return &wrappersServiceClient{cc}
}

func (c *wrappersServiceClient) Create(ctx context.Context, in *Wrappers, opts ...grpc.CallOption) (*Wrappers, error) {
	out := new(Wrappers)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/Create", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateStringValue(ctx context.Context, in *wrappers.StringValue, opts ...grpc.CallOption) (*wrappers.StringValue, error) {
	out := new(wrappers.StringValue)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateStringValue", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateInt32Value(ctx context.Context, in *wrappers.Int32Value, opts ...grpc.CallOption) (*wrappers.Int32Value, error) {
	out := new(wrappers.Int32Value)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateInt32Value", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateInt64Value(ctx context.Context, in *wrappers.Int64Value, opts ...grpc.CallOption) (*wrappers.Int64Value, error) {
	out := new(wrappers.Int64Value)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateInt64Value", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateFloatValue(ctx context.Context, in *wrappers.FloatValue, opts ...grpc.CallOption) (*wrappers.FloatValue, error) {
	out := new(wrappers.FloatValue)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateFloatValue", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateDoubleValue(ctx context.Context, in *wrappers.DoubleValue, opts ...grpc.CallOption) (*wrappers.DoubleValue, error) {
	out := new(wrappers.DoubleValue)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateDoubleValue", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateBoolValue(ctx context.Context, in *wrappers.BoolValue, opts ...grpc.CallOption) (*wrappers.BoolValue, error) {
	out := new(wrappers.BoolValue)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateBoolValue", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateUInt32Value(ctx context.Context, in *wrappers.UInt32Value, opts ...grpc.CallOption) (*wrappers.UInt32Value, error) {
	out := new(wrappers.UInt32Value)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateUInt32Value", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateUInt64Value(ctx context.Context, in *wrappers.UInt64Value, opts ...grpc.CallOption) (*wrappers.UInt64Value, error) {
	out := new(wrappers.UInt64Value)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateUInt64Value", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateBytesValue(ctx context.Context, in *wrappers.BytesValue, opts ...grpc.CallOption) (*wrappers.BytesValue, error) {
	out := new(wrappers.BytesValue)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateBytesValue", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *wrappersServiceClient) CreateEmpty(ctx context.Context, in *empty.Empty, opts ...grpc.CallOption) (*empty.Empty, error) {
	out := new(empty.Empty)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateEmpty", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

// WrappersServiceServer is the server API for WrappersService service.
type WrappersServiceServer interface {
	Create(context.Context, *Wrappers) (*Wrappers, error)
	CreateStringValue(context.Context, *wrappers.StringValue) (*wrappers.StringValue, error)
	CreateInt32Value(context.Context, *wrappers.Int32Value) (*wrappers.Int32Value, error)
	CreateInt64Value(context.Context, *wrappers.Int64Value) (*wrappers.Int64Value, error)
	CreateFloatValue(context.Context, *wrappers.FloatValue) (*wrappers.FloatValue, error)
	CreateDoubleValue(context.Context, *wrappers.DoubleValue) (*wrappers.DoubleValue, error)
	CreateBoolValue(context.Context, *wrappers.BoolValue) (*wrappers.BoolValue, error)
	CreateUInt32Value(context.Context, *wrappers.UInt32Value) (*wrappers.UInt32Value, error)
	CreateUInt64Value(context.Context, *wrappers.UInt64Value) (*wrappers.UInt64Value, error)
	CreateBytesValue(context.Context, *wrappers.BytesValue) (*wrappers.BytesValue, error)
	CreateEmpty(context.Context, *empty.Empty) (*empty.Empty, error)
}

// UnimplementedWrappersServiceServer can be embedded to have forward compatible implementations.
type UnimplementedWrappersServiceServer struct {
}

func (*UnimplementedWrappersServiceServer) Create(ctx context.Context, req *Wrappers) (*Wrappers, error) {
	return nil, status.Errorf(codes.Unimplemented, "method Create not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateStringValue(ctx context.Context, req *wrappers.StringValue) (*wrappers.StringValue, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateStringValue not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateInt32Value(ctx context.Context, req *wrappers.Int32Value) (*wrappers.Int32Value, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateInt32Value not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateInt64Value(ctx context.Context, req *wrappers.Int64Value) (*wrappers.Int64Value, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateInt64Value not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateFloatValue(ctx context.Context, req *wrappers.FloatValue) (*wrappers.FloatValue, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateFloatValue not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateDoubleValue(ctx context.Context, req *wrappers.DoubleValue) (*wrappers.DoubleValue, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateDoubleValue not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateBoolValue(ctx context.Context, req *wrappers.BoolValue) (*wrappers.BoolValue, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateBoolValue not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateUInt32Value(ctx context.Context, req *wrappers.UInt32Value) (*wrappers.UInt32Value, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateUInt32Value not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateUInt64Value(ctx context.Context, req *wrappers.UInt64Value) (*wrappers.UInt64Value, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateUInt64Value not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateBytesValue(ctx context.Context, req *wrappers.BytesValue) (*wrappers.BytesValue, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateBytesValue not implemented")
}
func (*UnimplementedWrappersServiceServer) CreateEmpty(ctx context.Context, req *empty.Empty) (*empty.Empty, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateEmpty not implemented")
}

func RegisterWrappersServiceServer(s *grpc.Server, srv WrappersServiceServer) {
	s.RegisterService(&_WrappersService_serviceDesc, srv)
}

func _WrappersService_Create_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(Wrappers)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).Create(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/Create",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).Create(ctx, req.(*Wrappers))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateStringValue_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(wrappers.StringValue)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateStringValue(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateStringValue",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateStringValue(ctx, req.(*wrappers.StringValue))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateInt32Value_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(wrappers.Int32Value)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateInt32Value(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateInt32Value",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateInt32Value(ctx, req.(*wrappers.Int32Value))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateInt64Value_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(wrappers.Int64Value)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateInt64Value(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateInt64Value",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateInt64Value(ctx, req.(*wrappers.Int64Value))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateFloatValue_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(wrappers.FloatValue)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateFloatValue(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateFloatValue",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateFloatValue(ctx, req.(*wrappers.FloatValue))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateDoubleValue_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(wrappers.DoubleValue)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateDoubleValue(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateDoubleValue",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateDoubleValue(ctx, req.(*wrappers.DoubleValue))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateBoolValue_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(wrappers.BoolValue)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateBoolValue(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateBoolValue",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateBoolValue(ctx, req.(*wrappers.BoolValue))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateUInt32Value_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(wrappers.UInt32Value)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateUInt32Value(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateUInt32Value",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateUInt32Value(ctx, req.(*wrappers.UInt32Value))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateUInt64Value_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(wrappers.UInt64Value)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateUInt64Value(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateUInt64Value",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateUInt64Value(ctx, req.(*wrappers.UInt64Value))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateBytesValue_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(wrappers.BytesValue)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateBytesValue(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateBytesValue",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateBytesValue(ctx, req.(*wrappers.BytesValue))
	}
	return interceptor(ctx, in, info, handler)
}

func _WrappersService_CreateEmpty_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(empty.Empty)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(WrappersServiceServer).CreateEmpty(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.WrappersService/CreateEmpty",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(WrappersServiceServer).CreateEmpty(ctx, req.(*empty.Empty))
	}
	return interceptor(ctx, in, info, handler)
}

var _WrappersService_serviceDesc = grpc.ServiceDesc{
	ServiceName: "grpc.gateway.examples.internal.examplepb.WrappersService",
	HandlerType: (*WrappersServiceServer)(nil),
	Methods: []grpc.MethodDesc{
		{
			MethodName: "Create",
			Handler:    _WrappersService_Create_Handler,
		},
		{
			MethodName: "CreateStringValue",
			Handler:    _WrappersService_CreateStringValue_Handler,
		},
		{
			MethodName: "CreateInt32Value",
			Handler:    _WrappersService_CreateInt32Value_Handler,
		},
		{
			MethodName: "CreateInt64Value",
			Handler:    _WrappersService_CreateInt64Value_Handler,
		},
		{
			MethodName: "CreateFloatValue",
			Handler:    _WrappersService_CreateFloatValue_Handler,
		},
		{
			MethodName: "CreateDoubleValue",
			Handler:    _WrappersService_CreateDoubleValue_Handler,
		},
		{
			MethodName: "CreateBoolValue",
			Handler:    _WrappersService_CreateBoolValue_Handler,
		},
		{
			MethodName: "CreateUInt32Value",
			Handler:    _WrappersService_CreateUInt32Value_Handler,
		},
		{
			MethodName: "CreateUInt64Value",
			Handler:    _WrappersService_CreateUInt64Value_Handler,
		},
		{
			MethodName: "CreateBytesValue",
			Handler:    _WrappersService_CreateBytesValue_Handler,
		},
		{
			MethodName: "CreateEmpty",
			Handler:    _WrappersService_CreateEmpty_Handler,
		},
	},
	Streams:  []grpc.StreamDesc{},
	Metadata: "examples/internal/proto/examplepb/wrappers.proto",
}
