// Code generated by protoc-gen-go. DO NOT EDIT.
// source: examples/internal/proto/examplepb/generate_unbound_methods.proto

// Generate Unannotated Methods Echo Service
// Similar to echo_service.proto but without annotations and without external configuration.
//
// Generate Unannotated Methods Echo Service API consists of a single service which returns
// a message.

package examplepb

import (
	context "context"
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	duration "github.com/golang/protobuf/ptypes/duration"
	grpc "google.golang.org/grpc"
	codes "google.golang.org/grpc/codes"
	status "google.golang.org/grpc/status"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

// GenerateUnboundMethodsSimpleMessage represents a simple message sent to the unannotated GenerateUnboundMethodsEchoService service.
type GenerateUnboundMethodsSimpleMessage struct {
	// Id represents the message identifier.
	Id                   string             `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	Num                  int64              `protobuf:"varint,2,opt,name=num,proto3" json:"num,omitempty"`
	Duration             *duration.Duration `protobuf:"bytes,3,opt,name=duration,proto3" json:"duration,omitempty"`
	XXX_NoUnkeyedLiteral struct{}           `json:"-"`
	XXX_unrecognized     []byte             `json:"-"`
	XXX_sizecache        int32              `json:"-"`
}

func (m *GenerateUnboundMethodsSimpleMessage) Reset()         { *m = GenerateUnboundMethodsSimpleMessage{} }
func (m *GenerateUnboundMethodsSimpleMessage) String() string { return proto.CompactTextString(m) }
func (*GenerateUnboundMethodsSimpleMessage) ProtoMessage()    {}
func (*GenerateUnboundMethodsSimpleMessage) Descriptor() ([]byte, []int) {
	return fileDescriptor_f1e157236816fdee, []int{0}
}

func (m *GenerateUnboundMethodsSimpleMessage) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_GenerateUnboundMethodsSimpleMessage.Unmarshal(m, b)
}
func (m *GenerateUnboundMethodsSimpleMessage) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_GenerateUnboundMethodsSimpleMessage.Marshal(b, m, deterministic)
}
func (m *GenerateUnboundMethodsSimpleMessage) XXX_Merge(src proto.Message) {
	xxx_messageInfo_GenerateUnboundMethodsSimpleMessage.Merge(m, src)
}
func (m *GenerateUnboundMethodsSimpleMessage) XXX_Size() int {
	return xxx_messageInfo_GenerateUnboundMethodsSimpleMessage.Size(m)
}
func (m *GenerateUnboundMethodsSimpleMessage) XXX_DiscardUnknown() {
	xxx_messageInfo_GenerateUnboundMethodsSimpleMessage.DiscardUnknown(m)
}

var xxx_messageInfo_GenerateUnboundMethodsSimpleMessage proto.InternalMessageInfo

func (m *GenerateUnboundMethodsSimpleMessage) GetId() string {
	if m != nil {
		return m.Id
	}
	return ""
}

func (m *GenerateUnboundMethodsSimpleMessage) GetNum() int64 {
	if m != nil {
		return m.Num
	}
	return 0
}

func (m *GenerateUnboundMethodsSimpleMessage) GetDuration() *duration.Duration {
	if m != nil {
		return m.Duration
	}
	return nil
}

func init() {
	proto.RegisterType((*GenerateUnboundMethodsSimpleMessage)(nil), "grpc.gateway.examples.internal.examplepb.GenerateUnboundMethodsSimpleMessage")
}

func init() {
	proto.RegisterFile("examples/internal/proto/examplepb/generate_unbound_methods.proto", fileDescriptor_f1e157236816fdee)
}

var fileDescriptor_f1e157236816fdee = []byte{
	// 288 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xcc, 0x92, 0xbd, 0x4e, 0xf3, 0x30,
	0x14, 0x86, 0xe5, 0xa4, 0xfa, 0xd4, 0xba, 0xd2, 0x27, 0xe4, 0x29, 0x74, 0x40, 0x01, 0x96, 0x4c,
	0x27, 0x52, 0x11, 0x3b, 0xaa, 0x8a, 0x98, 0xb2, 0xa4, 0x62, 0x61, 0xa9, 0x9c, 0xf8, 0xe0, 0x5a,
	0x4a, 0xec, 0xc8, 0x71, 0x80, 0x2e, 0x5c, 0x0c, 0x62, 0x40, 0x5c, 0x25, 0x4a, 0xf3, 0x33, 0x31,
	0x74, 0x60, 0xe8, 0x66, 0x5b, 0xaf, 0x9f, 0xe7, 0x3d, 0xd2, 0xa1, 0x77, 0xf8, 0xc6, 0xcb, 0xaa,
	0xc0, 0x3a, 0x56, 0xda, 0xa1, 0xd5, 0xbc, 0x88, 0x2b, 0x6b, 0x9c, 0x89, 0xfb, 0xf7, 0x2a, 0x8b,
	0x25, 0x6a, 0xb4, 0xdc, 0xe1, 0xb6, 0xd1, 0x99, 0x69, 0xb4, 0xd8, 0x96, 0xe8, 0x76, 0x46, 0xd4,
	0x70, 0x08, 0xb2, 0x48, 0xda, 0x2a, 0x07, 0xc9, 0x1d, 0xbe, 0xf2, 0x3d, 0x0c, 0x38, 0x18, 0x70,
	0x30, 0x82, 0x16, 0x17, 0xd2, 0x18, 0x59, 0x60, 0x27, 0xc8, 0x9a, 0xe7, 0x58, 0x34, 0x96, 0x3b,
	0x65, 0x74, 0x47, 0xba, 0x7a, 0xa7, 0xd7, 0x0f, 0xbd, 0xeb, 0xb1, 0x53, 0x25, 0x9d, 0x69, 0xa3,
	0x5a, 0x42, 0x82, 0x75, 0xcd, 0x25, 0xb2, 0xff, 0xd4, 0x53, 0x22, 0x20, 0x21, 0x89, 0x66, 0xa9,
	0xa7, 0x04, 0x3b, 0xa3, 0xbe, 0x6e, 0xca, 0xc0, 0x0b, 0x49, 0xe4, 0xa7, 0xed, 0x91, 0xdd, 0xd2,
	0xe9, 0x80, 0x0e, 0xfc, 0x90, 0x44, 0xf3, 0xe5, 0x39, 0x74, 0x6e, 0x18, 0xdc, 0xb0, 0xee, 0x03,
	0xe9, 0x18, 0x5d, 0x7e, 0x4c, 0xe8, 0xe5, 0xef, 0x05, 0xee, 0xf3, 0x9d, 0xd9, 0xa0, 0x7d, 0x51,
	0x39, 0xb2, 0x4f, 0x42, 0x27, 0xed, 0x9d, 0x25, 0x70, 0xec, 0xe4, 0x70, 0xc4, 0x58, 0x8b, 0xbf,
	0xc5, 0xb1, 0x2f, 0x42, 0xa7, 0x6d, 0xcd, 0x95, 0x11, 0xfb, 0x13, 0xaf, 0xfa, 0x4d, 0x28, 0x6d,
	0xab, 0xae, 0xb1, 0x40, 0x87, 0xa7, 0x5d, 0x76, 0x35, 0x7f, 0x9a, 0x8d, 0x1f, 0xb2, 0x7f, 0x87,
	0x75, 0xba, 0xf9, 0x09, 0x00, 0x00, 0xff, 0xff, 0x8d, 0x4c, 0x2e, 0x6a, 0x46, 0x03, 0x00, 0x00,
}

// Reference imports to suppress errors if they are not otherwise used.
var _ context.Context
var _ grpc.ClientConnInterface

// This is a compile-time assertion to ensure that this generated file
// is compatible with the grpc package it is being compiled against.
const _ = grpc.SupportPackageIsVersion6

// GenerateUnboundMethodsEchoServiceClient is the client API for GenerateUnboundMethodsEchoService service.
//
// For semantics around ctx use and closing/ending streaming RPCs, please refer to https://godoc.org/google.golang.org/grpc#ClientConn.NewStream.
type GenerateUnboundMethodsEchoServiceClient interface {
	// Echo method receives a simple message and returns it.
	//
	// The message posted as the id parameter will also be
	// returned.
	Echo(ctx context.Context, in *GenerateUnboundMethodsSimpleMessage, opts ...grpc.CallOption) (*GenerateUnboundMethodsSimpleMessage, error)
	// EchoBody method receives a simple message and returns it.
	EchoBody(ctx context.Context, in *GenerateUnboundMethodsSimpleMessage, opts ...grpc.CallOption) (*GenerateUnboundMethodsSimpleMessage, error)
	// EchoDelete method receives a simple message and returns it.
	EchoDelete(ctx context.Context, in *GenerateUnboundMethodsSimpleMessage, opts ...grpc.CallOption) (*GenerateUnboundMethodsSimpleMessage, error)
}

type generateUnboundMethodsEchoServiceClient struct {
	cc grpc.ClientConnInterface
}

func NewGenerateUnboundMethodsEchoServiceClient(cc grpc.ClientConnInterface) GenerateUnboundMethodsEchoServiceClient {
	return &generateUnboundMethodsEchoServiceClient{cc}
}

func (c *generateUnboundMethodsEchoServiceClient) Echo(ctx context.Context, in *GenerateUnboundMethodsSimpleMessage, opts ...grpc.CallOption) (*GenerateUnboundMethodsSimpleMessage, error) {
	out := new(GenerateUnboundMethodsSimpleMessage)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.GenerateUnboundMethodsEchoService/Echo", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *generateUnboundMethodsEchoServiceClient) EchoBody(ctx context.Context, in *GenerateUnboundMethodsSimpleMessage, opts ...grpc.CallOption) (*GenerateUnboundMethodsSimpleMessage, error) {
	out := new(GenerateUnboundMethodsSimpleMessage)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.GenerateUnboundMethodsEchoService/EchoBody", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *generateUnboundMethodsEchoServiceClient) EchoDelete(ctx context.Context, in *GenerateUnboundMethodsSimpleMessage, opts ...grpc.CallOption) (*GenerateUnboundMethodsSimpleMessage, error) {
	out := new(GenerateUnboundMethodsSimpleMessage)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.GenerateUnboundMethodsEchoService/EchoDelete", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

// GenerateUnboundMethodsEchoServiceServer is the server API for GenerateUnboundMethodsEchoService service.
type GenerateUnboundMethodsEchoServiceServer interface {
	// Echo method receives a simple message and returns it.
	//
	// The message posted as the id parameter will also be
	// returned.
	Echo(context.Context, *GenerateUnboundMethodsSimpleMessage) (*GenerateUnboundMethodsSimpleMessage, error)
	// EchoBody method receives a simple message and returns it.
	EchoBody(context.Context, *GenerateUnboundMethodsSimpleMessage) (*GenerateUnboundMethodsSimpleMessage, error)
	// EchoDelete method receives a simple message and returns it.
	EchoDelete(context.Context, *GenerateUnboundMethodsSimpleMessage) (*GenerateUnboundMethodsSimpleMessage, error)
}

// UnimplementedGenerateUnboundMethodsEchoServiceServer can be embedded to have forward compatible implementations.
type UnimplementedGenerateUnboundMethodsEchoServiceServer struct {
}

func (*UnimplementedGenerateUnboundMethodsEchoServiceServer) Echo(ctx context.Context, req *GenerateUnboundMethodsSimpleMessage) (*GenerateUnboundMethodsSimpleMessage, error) {
	return nil, status.Errorf(codes.Unimplemented, "method Echo not implemented")
}
func (*UnimplementedGenerateUnboundMethodsEchoServiceServer) EchoBody(ctx context.Context, req *GenerateUnboundMethodsSimpleMessage) (*GenerateUnboundMethodsSimpleMessage, error) {
	return nil, status.Errorf(codes.Unimplemented, "method EchoBody not implemented")
}
func (*UnimplementedGenerateUnboundMethodsEchoServiceServer) EchoDelete(ctx context.Context, req *GenerateUnboundMethodsSimpleMessage) (*GenerateUnboundMethodsSimpleMessage, error) {
	return nil, status.Errorf(codes.Unimplemented, "method EchoDelete not implemented")
}

func RegisterGenerateUnboundMethodsEchoServiceServer(s *grpc.Server, srv GenerateUnboundMethodsEchoServiceServer) {
	s.RegisterService(&_GenerateUnboundMethodsEchoService_serviceDesc, srv)
}

func _GenerateUnboundMethodsEchoService_Echo_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(GenerateUnboundMethodsSimpleMessage)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(GenerateUnboundMethodsEchoServiceServer).Echo(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.GenerateUnboundMethodsEchoService/Echo",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(GenerateUnboundMethodsEchoServiceServer).Echo(ctx, req.(*GenerateUnboundMethodsSimpleMessage))
	}
	return interceptor(ctx, in, info, handler)
}

func _GenerateUnboundMethodsEchoService_EchoBody_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(GenerateUnboundMethodsSimpleMessage)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(GenerateUnboundMethodsEchoServiceServer).EchoBody(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.GenerateUnboundMethodsEchoService/EchoBody",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(GenerateUnboundMethodsEchoServiceServer).EchoBody(ctx, req.(*GenerateUnboundMethodsSimpleMessage))
	}
	return interceptor(ctx, in, info, handler)
}

func _GenerateUnboundMethodsEchoService_EchoDelete_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(GenerateUnboundMethodsSimpleMessage)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(GenerateUnboundMethodsEchoServiceServer).EchoDelete(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.GenerateUnboundMethodsEchoService/EchoDelete",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(GenerateUnboundMethodsEchoServiceServer).EchoDelete(ctx, req.(*GenerateUnboundMethodsSimpleMessage))
	}
	return interceptor(ctx, in, info, handler)
}

var _GenerateUnboundMethodsEchoService_serviceDesc = grpc.ServiceDesc{
	ServiceName: "grpc.gateway.examples.internal.examplepb.GenerateUnboundMethodsEchoService",
	HandlerType: (*GenerateUnboundMethodsEchoServiceServer)(nil),
	Methods: []grpc.MethodDesc{
		{
			MethodName: "Echo",
			Handler:    _GenerateUnboundMethodsEchoService_Echo_Handler,
		},
		{
			MethodName: "EchoBody",
			Handler:    _GenerateUnboundMethodsEchoService_EchoBody_Handler,
		},
		{
			MethodName: "EchoDelete",
			Handler:    _GenerateUnboundMethodsEchoService_EchoDelete_Handler,
		},
	},
	Streams:  []grpc.StreamDesc{},
	Metadata: "examples/internal/proto/examplepb/generate_unbound_methods.proto",
}
