// Code generated by protoc-gen-go. DO NOT EDIT.
// source: examples/internal/helloworld/helloworld.proto

package helloworld

import (
	context "context"
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	wrappers "github.com/golang/protobuf/ptypes/wrappers"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	grpc "google.golang.org/grpc"
	codes "google.golang.org/grpc/codes"
	status "google.golang.org/grpc/status"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type HelloRequest struct {
	Name                 string                `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	StrVal               *wrappers.StringValue `protobuf:"bytes,2,opt,name=strVal,proto3" json:"strVal,omitempty"`
	FloatVal             *wrappers.FloatValue  `protobuf:"bytes,3,opt,name=floatVal,proto3" json:"floatVal,omitempty"`
	DoubleVal            *wrappers.DoubleValue `protobuf:"bytes,4,opt,name=doubleVal,proto3" json:"doubleVal,omitempty"`
	BoolVal              *wrappers.BoolValue   `protobuf:"bytes,5,opt,name=boolVal,proto3" json:"boolVal,omitempty"`
	BytesVal             *wrappers.BytesValue  `protobuf:"bytes,6,opt,name=bytesVal,proto3" json:"bytesVal,omitempty"`
	Int32Val             *wrappers.Int32Value  `protobuf:"bytes,7,opt,name=int32Val,proto3" json:"int32Val,omitempty"`
	Uint32Val            *wrappers.UInt32Value `protobuf:"bytes,8,opt,name=uint32Val,proto3" json:"uint32Val,omitempty"`
	Int64Val             *wrappers.Int64Value  `protobuf:"bytes,9,opt,name=int64Val,proto3" json:"int64Val,omitempty"`
	Uint64Val            *wrappers.UInt64Value `protobuf:"bytes,10,opt,name=uint64Val,proto3" json:"uint64Val,omitempty"`
	XXX_NoUnkeyedLiteral struct{}              `json:"-"`
	XXX_unrecognized     []byte                `json:"-"`
	XXX_sizecache        int32                 `json:"-"`
}

func (m *HelloRequest) Reset()         { *m = HelloRequest{} }
func (m *HelloRequest) String() string { return proto.CompactTextString(m) }
func (*HelloRequest) ProtoMessage()    {}
func (*HelloRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_fd8fde893e1fcf9f, []int{0}
}

func (m *HelloRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_HelloRequest.Unmarshal(m, b)
}
func (m *HelloRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_HelloRequest.Marshal(b, m, deterministic)
}
func (m *HelloRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_HelloRequest.Merge(m, src)
}
func (m *HelloRequest) XXX_Size() int {
	return xxx_messageInfo_HelloRequest.Size(m)
}
func (m *HelloRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_HelloRequest.DiscardUnknown(m)
}

var xxx_messageInfo_HelloRequest proto.InternalMessageInfo

func (m *HelloRequest) GetName() string {
	if m != nil {
		return m.Name
	}
	return ""
}

func (m *HelloRequest) GetStrVal() *wrappers.StringValue {
	if m != nil {
		return m.StrVal
	}
	return nil
}

func (m *HelloRequest) GetFloatVal() *wrappers.FloatValue {
	if m != nil {
		return m.FloatVal
	}
	return nil
}

func (m *HelloRequest) GetDoubleVal() *wrappers.DoubleValue {
	if m != nil {
		return m.DoubleVal
	}
	return nil
}

func (m *HelloRequest) GetBoolVal() *wrappers.BoolValue {
	if m != nil {
		return m.BoolVal
	}
	return nil
}

func (m *HelloRequest) GetBytesVal() *wrappers.BytesValue {
	if m != nil {
		return m.BytesVal
	}
	return nil
}

func (m *HelloRequest) GetInt32Val() *wrappers.Int32Value {
	if m != nil {
		return m.Int32Val
	}
	return nil
}

func (m *HelloRequest) GetUint32Val() *wrappers.UInt32Value {
	if m != nil {
		return m.Uint32Val
	}
	return nil
}

func (m *HelloRequest) GetInt64Val() *wrappers.Int64Value {
	if m != nil {
		return m.Int64Val
	}
	return nil
}

func (m *HelloRequest) GetUint64Val() *wrappers.UInt64Value {
	if m != nil {
		return m.Uint64Val
	}
	return nil
}

type HelloReply struct {
	Message              string   `protobuf:"bytes,1,opt,name=message,proto3" json:"message,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *HelloReply) Reset()         { *m = HelloReply{} }
func (m *HelloReply) String() string { return proto.CompactTextString(m) }
func (*HelloReply) ProtoMessage()    {}
func (*HelloReply) Descriptor() ([]byte, []int) {
	return fileDescriptor_fd8fde893e1fcf9f, []int{1}
}

func (m *HelloReply) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_HelloReply.Unmarshal(m, b)
}
func (m *HelloReply) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_HelloReply.Marshal(b, m, deterministic)
}
func (m *HelloReply) XXX_Merge(src proto.Message) {
	xxx_messageInfo_HelloReply.Merge(m, src)
}
func (m *HelloReply) XXX_Size() int {
	return xxx_messageInfo_HelloReply.Size(m)
}
func (m *HelloReply) XXX_DiscardUnknown() {
	xxx_messageInfo_HelloReply.DiscardUnknown(m)
}

var xxx_messageInfo_HelloReply proto.InternalMessageInfo

func (m *HelloReply) GetMessage() string {
	if m != nil {
		return m.Message
	}
	return ""
}

func init() {
	proto.RegisterType((*HelloRequest)(nil), "helloworld.HelloRequest")
	proto.RegisterType((*HelloReply)(nil), "helloworld.HelloReply")
}

func init() {
	proto.RegisterFile("examples/internal/helloworld/helloworld.proto", fileDescriptor_fd8fde893e1fcf9f)
}

var fileDescriptor_fd8fde893e1fcf9f = []byte{
	// 482 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x7c, 0x92, 0x3d, 0x6f, 0xd4, 0x30,
	0x1c, 0xc6, 0x75, 0xe1, 0xb8, 0x17, 0x97, 0xc9, 0x42, 0xa7, 0x28, 0x9c, 0x50, 0x75, 0x03, 0xea,
	0x42, 0x22, 0x5d, 0x4f, 0x45, 0x62, 0xac, 0x10, 0x2f, 0x6b, 0x2a, 0x3a, 0xdc, 0xe6, 0xa8, 0xff,
	0x1e, 0x91, 0x7c, 0x71, 0xf0, 0x0b, 0x25, 0xaa, 0xb2, 0xb0, 0xb1, 0x21, 0xf1, 0x1d, 0xf8, 0x3a,
	0x0c, 0x8c, 0xac, 0x7c, 0x10, 0xe4, 0x7f, 0xec, 0x73, 0x50, 0x49, 0x37, 0x3b, 0xcf, 0xf3, 0xf3,
	0x63, 0xe7, 0xff, 0x90, 0xe7, 0xf0, 0x99, 0xed, 0x6b, 0x0e, 0x2a, 0x2b, 0x2b, 0x0d, 0xb2, 0x62,
	0x3c, 0xfb, 0x00, 0x9c, 0x8b, 0x1b, 0x21, 0xf9, 0x55, 0x6f, 0x99, 0xd6, 0x52, 0x68, 0x41, 0x49,
	0xf8, 0x92, 0x2c, 0x77, 0x42, 0xec, 0x38, 0x64, 0xac, 0x2e, 0x33, 0x56, 0x55, 0x42, 0x33, 0x5d,
	0x8a, 0x4a, 0x75, 0xce, 0xe4, 0xa9, 0x53, 0x71, 0x57, 0x98, 0xeb, 0xec, 0x46, 0xb2, 0xba, 0x06,
	0xe9, 0xf4, 0xd5, 0x8f, 0x31, 0x79, 0xf4, 0xd6, 0x1e, 0x96, 0xc3, 0x47, 0x03, 0x4a, 0x53, 0x4a,
	0xc6, 0x15, 0xdb, 0x43, 0x3c, 0x3a, 0x1e, 0x9d, 0xcc, 0x73, 0x5c, 0xd3, 0x0d, 0x99, 0x28, 0x2d,
	0x2f, 0x19, 0x8f, 0xa3, 0xe3, 0xd1, 0xc9, 0xd1, 0x7a, 0x99, 0x76, 0xa7, 0xa6, 0xfe, 0xd4, 0xf4,
	0x42, 0xcb, 0xb2, 0xda, 0x5d, 0x32, 0x6e, 0x20, 0x77, 0x5e, 0xfa, 0x82, 0xcc, 0xae, 0xb9, 0x60,
	0xda, 0x72, 0x0f, 0x90, 0x7b, 0x72, 0x87, 0x7b, 0xed, 0x0c, 0x06, 0xf2, 0x83, 0x99, 0xbe, 0x24,
	0xf3, 0x2b, 0x61, 0x0a, 0x0e, 0x96, 0x1c, 0x0f, 0x24, 0xbe, 0xf2, 0x0e, 0x03, 0x79, 0xb0, 0xd3,
	0x0d, 0x99, 0x16, 0x42, 0x70, 0x4b, 0x3e, 0x44, 0x32, 0xb9, 0x43, 0x9e, 0x77, 0xba, 0x81, 0xdc,
	0x5b, 0xed, 0x55, 0x8b, 0x46, 0x83, 0xb2, 0xd8, 0x64, 0xe0, 0xaa, 0xe7, 0xce, 0x60, 0xaf, 0xea,
	0xcd, 0x16, 0x2c, 0x2b, 0x7d, 0xba, 0xb6, 0xe0, 0x74, 0x00, 0x7c, 0xe7, 0x0c, 0x16, 0xf4, 0x66,
	0xfb, 0x46, 0x73, 0x20, 0x67, 0x03, 0x6f, 0x7c, 0xdf, 0x43, 0x83, 0xdd, 0x85, 0x9e, 0x6d, 0x2c,
	0x3a, 0x1f, 0x0e, 0x45, 0x83, 0x0b, 0xc5, 0xb5, 0x0f, 0xed, 0x48, 0x72, 0x4f, 0xa8, 0x47, 0x83,
	0x7d, 0xf5, 0x8c, 0x10, 0xd7, 0x93, 0x9a, 0x37, 0x34, 0x26, 0xd3, 0x3d, 0x28, 0xc5, 0x76, 0xbe,
	0x28, 0x7e, 0xbb, 0xfe, 0x1d, 0x91, 0xe9, 0x1b, 0x09, 0xa0, 0x41, 0xd2, 0x9f, 0x11, 0x99, 0x5d,
	0xb0, 0x06, 0x39, 0x1a, 0xa7, 0xbd, 0x1a, 0xf7, 0x2b, 0x97, 0x2c, 0xfe, 0xa3, 0xd4, 0xbc, 0x59,
	0x7d, 0x8b, 0xbe, 0xfc, 0xfa, 0xf3, 0x3d, 0xfa, 0x1a, 0xd1, 0xa3, 0x4c, 0xb1, 0x26, 0xbb, 0xb5,
	0x65, 0x6c, 0xb7, 0x0b, 0xfa, 0x18, 0xb7, 0x4a, 0xcb, 0x4f, 0x8c, 0x67, 0xb7, 0x5d, 0xdd, 0xda,
	0x6d, 0x42, 0x63, 0xfc, 0x8e, 0x3d, 0x42, 0xc5, 0x37, 0xaa, 0xdd, 0x2e, 0x69, 0x82, 0x5a, 0x57,
	0x14, 0x14, 0x0f, 0x9d, 0x69, 0xb7, 0x31, 0x5d, 0xa0, 0x6a, 0xeb, 0x80, 0x9a, 0xeb, 0x45, 0x38,
	0x13, 0x07, 0xde, 0x49, 0x6e, 0xf4, 0x41, 0xc3, 0xb9, 0xa0, 0xe6, 0x27, 0x14, 0xf2, 0x4c, 0x10,
	0x4d, 0x50, 0x7b, 0xe4, 0xd9, 0xc6, 0x93, 0xf8, 0x9b, 0xff, 0x25, 0x9d, 0x78, 0x18, 0x42, 0x5b,
	0x4c, 0x70, 0x4c, 0xa7, 0x7f, 0x03, 0x00, 0x00, 0xff, 0xff, 0xca, 0x80, 0x5f, 0x77, 0x30, 0x04,
	0x00, 0x00,
}

// Reference imports to suppress errors if they are not otherwise used.
var _ context.Context
var _ grpc.ClientConnInterface

// This is a compile-time assertion to ensure that this generated file
// is compatible with the grpc package it is being compiled against.
const _ = grpc.SupportPackageIsVersion6

// GreeterClient is the client API for Greeter service.
//
// For semantics around ctx use and closing/ending streaming RPCs, please refer to https://godoc.org/google.golang.org/grpc#ClientConn.NewStream.
type GreeterClient interface {
	SayHello(ctx context.Context, in *HelloRequest, opts ...grpc.CallOption) (*HelloReply, error)
}

type greeterClient struct {
	cc grpc.ClientConnInterface
}

func NewGreeterClient(cc grpc.ClientConnInterface) GreeterClient {
	return &greeterClient{cc}
}

func (c *greeterClient) SayHello(ctx context.Context, in *HelloRequest, opts ...grpc.CallOption) (*HelloReply, error) {
	out := new(HelloReply)
	err := c.cc.Invoke(ctx, "/helloworld.Greeter/SayHello", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

// GreeterServer is the server API for Greeter service.
type GreeterServer interface {
	SayHello(context.Context, *HelloRequest) (*HelloReply, error)
}

// UnimplementedGreeterServer can be embedded to have forward compatible implementations.
type UnimplementedGreeterServer struct {
}

func (*UnimplementedGreeterServer) SayHello(ctx context.Context, req *HelloRequest) (*HelloReply, error) {
	return nil, status.Errorf(codes.Unimplemented, "method SayHello not implemented")
}

func RegisterGreeterServer(s *grpc.Server, srv GreeterServer) {
	s.RegisterService(&_Greeter_serviceDesc, srv)
}

func _Greeter_SayHello_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(HelloRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(GreeterServer).SayHello(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/helloworld.Greeter/SayHello",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(GreeterServer).SayHello(ctx, req.(*HelloRequest))
	}
	return interceptor(ctx, in, info, handler)
}

var _Greeter_serviceDesc = grpc.ServiceDesc{
	ServiceName: "helloworld.Greeter",
	HandlerType: (*GreeterServer)(nil),
	Methods: []grpc.MethodDesc{
		{
			MethodName: "SayHello",
			Handler:    _Greeter_SayHello_Handler,
		},
	},
	Streams:  []grpc.StreamDesc{},
	Metadata: "examples/internal/helloworld/helloworld.proto",
}
