// Code generated by protoc-gen-go. DO NOT EDIT.
// source: examples/internal/proto/examplepb/response_body_service.proto

package examplepb

import (
	context "context"
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	grpc "google.golang.org/grpc"
	codes "google.golang.org/grpc/codes"
	status "google.golang.org/grpc/status"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type RepeatedResponseBodyOut_Response_ResponseType int32

const (
	// UNKNOWN
	RepeatedResponseBodyOut_Response_UNKNOWN RepeatedResponseBodyOut_Response_ResponseType = 0
	// A is 1
	RepeatedResponseBodyOut_Response_A RepeatedResponseBodyOut_Response_ResponseType = 1
	// B is 2
	RepeatedResponseBodyOut_Response_B RepeatedResponseBodyOut_Response_ResponseType = 2
)

var RepeatedResponseBodyOut_Response_ResponseType_name = map[int32]string{
	0: "UNKNOWN",
	1: "A",
	2: "B",
}

var RepeatedResponseBodyOut_Response_ResponseType_value = map[string]int32{
	"UNKNOWN": 0,
	"A":       1,
	"B":       2,
}

func (x RepeatedResponseBodyOut_Response_ResponseType) String() string {
	return proto.EnumName(RepeatedResponseBodyOut_Response_ResponseType_name, int32(x))
}

func (RepeatedResponseBodyOut_Response_ResponseType) EnumDescriptor() ([]byte, []int) {
	return fileDescriptor_272b2870183bbe20, []int{2, 0, 0}
}

type ResponseBodyIn struct {
	Data                 string   `protobuf:"bytes,1,opt,name=data,proto3" json:"data,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *ResponseBodyIn) Reset()         { *m = ResponseBodyIn{} }
func (m *ResponseBodyIn) String() string { return proto.CompactTextString(m) }
func (*ResponseBodyIn) ProtoMessage()    {}
func (*ResponseBodyIn) Descriptor() ([]byte, []int) {
	return fileDescriptor_272b2870183bbe20, []int{0}
}

func (m *ResponseBodyIn) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ResponseBodyIn.Unmarshal(m, b)
}
func (m *ResponseBodyIn) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ResponseBodyIn.Marshal(b, m, deterministic)
}
func (m *ResponseBodyIn) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ResponseBodyIn.Merge(m, src)
}
func (m *ResponseBodyIn) XXX_Size() int {
	return xxx_messageInfo_ResponseBodyIn.Size(m)
}
func (m *ResponseBodyIn) XXX_DiscardUnknown() {
	xxx_messageInfo_ResponseBodyIn.DiscardUnknown(m)
}

var xxx_messageInfo_ResponseBodyIn proto.InternalMessageInfo

func (m *ResponseBodyIn) GetData() string {
	if m != nil {
		return m.Data
	}
	return ""
}

type ResponseBodyOut struct {
	Response             *ResponseBodyOut_Response `protobuf:"bytes,2,opt,name=response,proto3" json:"response,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                  `json:"-"`
	XXX_unrecognized     []byte                    `json:"-"`
	XXX_sizecache        int32                     `json:"-"`
}

func (m *ResponseBodyOut) Reset()         { *m = ResponseBodyOut{} }
func (m *ResponseBodyOut) String() string { return proto.CompactTextString(m) }
func (*ResponseBodyOut) ProtoMessage()    {}
func (*ResponseBodyOut) Descriptor() ([]byte, []int) {
	return fileDescriptor_272b2870183bbe20, []int{1}
}

func (m *ResponseBodyOut) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ResponseBodyOut.Unmarshal(m, b)
}
func (m *ResponseBodyOut) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ResponseBodyOut.Marshal(b, m, deterministic)
}
func (m *ResponseBodyOut) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ResponseBodyOut.Merge(m, src)
}
func (m *ResponseBodyOut) XXX_Size() int {
	return xxx_messageInfo_ResponseBodyOut.Size(m)
}
func (m *ResponseBodyOut) XXX_DiscardUnknown() {
	xxx_messageInfo_ResponseBodyOut.DiscardUnknown(m)
}

var xxx_messageInfo_ResponseBodyOut proto.InternalMessageInfo

func (m *ResponseBodyOut) GetResponse() *ResponseBodyOut_Response {
	if m != nil {
		return m.Response
	}
	return nil
}

type ResponseBodyOut_Response struct {
	Data                 string   `protobuf:"bytes,1,opt,name=data,proto3" json:"data,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *ResponseBodyOut_Response) Reset()         { *m = ResponseBodyOut_Response{} }
func (m *ResponseBodyOut_Response) String() string { return proto.CompactTextString(m) }
func (*ResponseBodyOut_Response) ProtoMessage()    {}
func (*ResponseBodyOut_Response) Descriptor() ([]byte, []int) {
	return fileDescriptor_272b2870183bbe20, []int{1, 0}
}

func (m *ResponseBodyOut_Response) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ResponseBodyOut_Response.Unmarshal(m, b)
}
func (m *ResponseBodyOut_Response) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ResponseBodyOut_Response.Marshal(b, m, deterministic)
}
func (m *ResponseBodyOut_Response) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ResponseBodyOut_Response.Merge(m, src)
}
func (m *ResponseBodyOut_Response) XXX_Size() int {
	return xxx_messageInfo_ResponseBodyOut_Response.Size(m)
}
func (m *ResponseBodyOut_Response) XXX_DiscardUnknown() {
	xxx_messageInfo_ResponseBodyOut_Response.DiscardUnknown(m)
}

var xxx_messageInfo_ResponseBodyOut_Response proto.InternalMessageInfo

func (m *ResponseBodyOut_Response) GetData() string {
	if m != nil {
		return m.Data
	}
	return ""
}

type RepeatedResponseBodyOut struct {
	Response             []*RepeatedResponseBodyOut_Response `protobuf:"bytes,2,rep,name=response,proto3" json:"response,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                            `json:"-"`
	XXX_unrecognized     []byte                              `json:"-"`
	XXX_sizecache        int32                               `json:"-"`
}

func (m *RepeatedResponseBodyOut) Reset()         { *m = RepeatedResponseBodyOut{} }
func (m *RepeatedResponseBodyOut) String() string { return proto.CompactTextString(m) }
func (*RepeatedResponseBodyOut) ProtoMessage()    {}
func (*RepeatedResponseBodyOut) Descriptor() ([]byte, []int) {
	return fileDescriptor_272b2870183bbe20, []int{2}
}

func (m *RepeatedResponseBodyOut) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_RepeatedResponseBodyOut.Unmarshal(m, b)
}
func (m *RepeatedResponseBodyOut) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_RepeatedResponseBodyOut.Marshal(b, m, deterministic)
}
func (m *RepeatedResponseBodyOut) XXX_Merge(src proto.Message) {
	xxx_messageInfo_RepeatedResponseBodyOut.Merge(m, src)
}
func (m *RepeatedResponseBodyOut) XXX_Size() int {
	return xxx_messageInfo_RepeatedResponseBodyOut.Size(m)
}
func (m *RepeatedResponseBodyOut) XXX_DiscardUnknown() {
	xxx_messageInfo_RepeatedResponseBodyOut.DiscardUnknown(m)
}

var xxx_messageInfo_RepeatedResponseBodyOut proto.InternalMessageInfo

func (m *RepeatedResponseBodyOut) GetResponse() []*RepeatedResponseBodyOut_Response {
	if m != nil {
		return m.Response
	}
	return nil
}

type RepeatedResponseBodyOut_Response struct {
	Data                 string                                        `protobuf:"bytes,1,opt,name=data,proto3" json:"data,omitempty"`
	Type                 RepeatedResponseBodyOut_Response_ResponseType `protobuf:"varint,3,opt,name=type,proto3,enum=grpc.gateway.examples.internal.examplepb.RepeatedResponseBodyOut_Response_ResponseType" json:"type,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                                      `json:"-"`
	XXX_unrecognized     []byte                                        `json:"-"`
	XXX_sizecache        int32                                         `json:"-"`
}

func (m *RepeatedResponseBodyOut_Response) Reset()         { *m = RepeatedResponseBodyOut_Response{} }
func (m *RepeatedResponseBodyOut_Response) String() string { return proto.CompactTextString(m) }
func (*RepeatedResponseBodyOut_Response) ProtoMessage()    {}
func (*RepeatedResponseBodyOut_Response) Descriptor() ([]byte, []int) {
	return fileDescriptor_272b2870183bbe20, []int{2, 0}
}

func (m *RepeatedResponseBodyOut_Response) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_RepeatedResponseBodyOut_Response.Unmarshal(m, b)
}
func (m *RepeatedResponseBodyOut_Response) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_RepeatedResponseBodyOut_Response.Marshal(b, m, deterministic)
}
func (m *RepeatedResponseBodyOut_Response) XXX_Merge(src proto.Message) {
	xxx_messageInfo_RepeatedResponseBodyOut_Response.Merge(m, src)
}
func (m *RepeatedResponseBodyOut_Response) XXX_Size() int {
	return xxx_messageInfo_RepeatedResponseBodyOut_Response.Size(m)
}
func (m *RepeatedResponseBodyOut_Response) XXX_DiscardUnknown() {
	xxx_messageInfo_RepeatedResponseBodyOut_Response.DiscardUnknown(m)
}

var xxx_messageInfo_RepeatedResponseBodyOut_Response proto.InternalMessageInfo

func (m *RepeatedResponseBodyOut_Response) GetData() string {
	if m != nil {
		return m.Data
	}
	return ""
}

func (m *RepeatedResponseBodyOut_Response) GetType() RepeatedResponseBodyOut_Response_ResponseType {
	if m != nil {
		return m.Type
	}
	return RepeatedResponseBodyOut_Response_UNKNOWN
}

type RepeatedResponseStrings struct {
	Values               []string `protobuf:"bytes,1,rep,name=values,proto3" json:"values,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *RepeatedResponseStrings) Reset()         { *m = RepeatedResponseStrings{} }
func (m *RepeatedResponseStrings) String() string { return proto.CompactTextString(m) }
func (*RepeatedResponseStrings) ProtoMessage()    {}
func (*RepeatedResponseStrings) Descriptor() ([]byte, []int) {
	return fileDescriptor_272b2870183bbe20, []int{3}
}

func (m *RepeatedResponseStrings) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_RepeatedResponseStrings.Unmarshal(m, b)
}
func (m *RepeatedResponseStrings) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_RepeatedResponseStrings.Marshal(b, m, deterministic)
}
func (m *RepeatedResponseStrings) XXX_Merge(src proto.Message) {
	xxx_messageInfo_RepeatedResponseStrings.Merge(m, src)
}
func (m *RepeatedResponseStrings) XXX_Size() int {
	return xxx_messageInfo_RepeatedResponseStrings.Size(m)
}
func (m *RepeatedResponseStrings) XXX_DiscardUnknown() {
	xxx_messageInfo_RepeatedResponseStrings.DiscardUnknown(m)
}

var xxx_messageInfo_RepeatedResponseStrings proto.InternalMessageInfo

func (m *RepeatedResponseStrings) GetValues() []string {
	if m != nil {
		return m.Values
	}
	return nil
}

func init() {
	proto.RegisterEnum("grpc.gateway.examples.internal.examplepb.RepeatedResponseBodyOut_Response_ResponseType", RepeatedResponseBodyOut_Response_ResponseType_name, RepeatedResponseBodyOut_Response_ResponseType_value)
	proto.RegisterType((*ResponseBodyIn)(nil), "grpc.gateway.examples.internal.examplepb.ResponseBodyIn")
	proto.RegisterType((*ResponseBodyOut)(nil), "grpc.gateway.examples.internal.examplepb.ResponseBodyOut")
	proto.RegisterType((*ResponseBodyOut_Response)(nil), "grpc.gateway.examples.internal.examplepb.ResponseBodyOut.Response")
	proto.RegisterType((*RepeatedResponseBodyOut)(nil), "grpc.gateway.examples.internal.examplepb.RepeatedResponseBodyOut")
	proto.RegisterType((*RepeatedResponseBodyOut_Response)(nil), "grpc.gateway.examples.internal.examplepb.RepeatedResponseBodyOut.Response")
	proto.RegisterType((*RepeatedResponseStrings)(nil), "grpc.gateway.examples.internal.examplepb.RepeatedResponseStrings")
}

func init() {
	proto.RegisterFile("examples/internal/proto/examplepb/response_body_service.proto", fileDescriptor_272b2870183bbe20)
}

var fileDescriptor_272b2870183bbe20 = []byte{
	// 482 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xc4, 0x94, 0xc1, 0x6e, 0xd3, 0x40,
	0x10, 0x86, 0xd9, 0xa4, 0x84, 0x66, 0x82, 0xda, 0x68, 0x0a, 0x6d, 0x14, 0x10, 0x32, 0x16, 0xa2,
	0xe6, 0x80, 0x0d, 0xe1, 0x02, 0x07, 0x0e, 0xcd, 0x05, 0x15, 0x50, 0x2a, 0x39, 0xa0, 0x4a, 0x1c,
	0xa8, 0x36, 0xcd, 0x60, 0x59, 0xa4, 0xbb, 0x2b, 0xef, 0xb6, 0x60, 0x21, 0x2e, 0x7d, 0x03, 0xc4,
	0x7b, 0x70, 0xe4, 0x00, 0xe2, 0x29, 0x78, 0x05, 0x1e, 0x04, 0xc5, 0xb1, 0x57, 0x6e, 0x6a, 0x21,
	0x68, 0x91, 0x72, 0xf2, 0xee, 0xec, 0xea, 0x9f, 0x4f, 0xfb, 0xff, 0x63, 0x78, 0x4c, 0xef, 0xf9,
	0x81, 0x9a, 0x90, 0x0e, 0x62, 0x61, 0x28, 0x11, 0x7c, 0x12, 0xa8, 0x44, 0x1a, 0x19, 0xe4, 0x75,
	0x35, 0x0a, 0x12, 0xd2, 0x4a, 0x0a, 0x4d, 0x7b, 0x23, 0x39, 0x4e, 0xf7, 0x34, 0x25, 0x47, 0xf1,
	0x3e, 0xf9, 0xd9, 0x2d, 0xf4, 0xa2, 0x44, 0xed, 0xfb, 0x11, 0x37, 0xf4, 0x8e, 0xa7, 0x7e, 0xa1,
	0xe5, 0x17, 0x5a, 0xbe, 0x55, 0xe9, 0x5e, 0x8f, 0xa4, 0x8c, 0x26, 0x14, 0x70, 0x15, 0x07, 0x5c,
	0x08, 0x69, 0xb8, 0x89, 0xa5, 0xd0, 0x33, 0x1d, 0xf7, 0x16, 0xac, 0x84, 0x79, 0x9b, 0xbe, 0x1c,
	0xa7, 0xdb, 0x02, 0x11, 0x96, 0xc6, 0xdc, 0xf0, 0x0e, 0x73, 0x98, 0xd7, 0x0c, 0xb3, 0xb5, 0xfb,
	0x89, 0xc1, 0x6a, 0xf9, 0xda, 0xce, 0xa1, 0xc1, 0xd7, 0xb0, 0x5c, 0x00, 0x76, 0x6a, 0x0e, 0xf3,
	0x5a, 0xbd, 0xbe, 0xff, 0xb7, 0x50, 0xfe, 0x9c, 0x98, 0xdd, 0x87, 0x56, 0xb3, 0x7b, 0x03, 0x96,
	0x8b, 0x6a, 0x25, 0xd3, 0xb7, 0x1a, 0x6c, 0x84, 0xa4, 0x88, 0x1b, 0x1a, 0xcf, 0xb3, 0xbd, 0x39,
	0xc1, 0x56, 0xf7, 0x5a, 0xbd, 0xa7, 0xff, 0xc2, 0x56, 0x29, 0x5a, 0xc5, 0xf8, 0x95, 0xfd, 0x19,
	0x12, 0xdf, 0xc2, 0x92, 0x49, 0x15, 0x75, 0xea, 0x0e, 0xf3, 0x56, 0x7a, 0xbb, 0xff, 0x0f, 0xc2,
	0x2e, 0x5e, 0xa4, 0x8a, 0xc2, 0xac, 0x89, 0x7b, 0x07, 0x2e, 0x97, 0xab, 0xd8, 0x82, 0x4b, 0x2f,
	0x07, 0xcf, 0x06, 0x3b, 0xbb, 0x83, 0xf6, 0x05, 0xbc, 0x08, 0x6c, 0xab, 0xcd, 0xa6, 0x9f, 0x7e,
	0xbb, 0xe6, 0xde, 0x3f, 0xfd, 0x76, 0x43, 0x93, 0xc4, 0x22, 0xd2, 0xb8, 0x0e, 0x8d, 0x23, 0x3e,
	0x39, 0x24, 0xdd, 0x61, 0x4e, 0xdd, 0x6b, 0x86, 0xf9, 0xae, 0x77, 0xdc, 0x80, 0xb5, 0x32, 0xcd,
	0x70, 0x96, 0x47, 0xfc, 0xc2, 0x60, 0xf5, 0x09, 0x99, 0xf2, 0x11, 0x3e, 0x3c, 0x5b, 0x12, 0xb6,
	0x45, 0xf7, 0xd1, 0x99, 0x33, 0xe4, 0xde, 0x3e, 0xfe, 0xf9, 0xeb, 0x73, 0xcd, 0xc1, 0x2b, 0x76,
	0x70, 0xa6, 0x73, 0x13, 0x7c, 0x98, 0x1a, 0xf1, 0x71, 0x64, 0x4d, 0xc3, 0xef, 0x0c, 0xf0, 0x79,
	0xac, 0xcb, 0xc4, 0x31, 0xe9, 0x73, 0x30, 0x6f, 0x9d, 0xdb, 0x56, 0xd7, 0xcb, 0xd8, 0x5d, 0x5c,
	0x2f, 0xb3, 0xc7, 0xa4, 0x2b, 0xe9, 0xd7, 0xca, 0xf4, 0x85, 0x6d, 0x0b, 0xc1, 0xcf, 0x9b, 0xbb,
	0x9b, 0x19, 0xfe, 0x4d, 0xdc, 0xb0, 0xf8, 0x7a, 0x76, 0x52, 0xf0, 0xe7, 0x21, 0xc2, 0x1f, 0x0c,
	0xae, 0xce, 0x85, 0x65, 0x68, 0x12, 0xe2, 0x07, 0x8b, 0x89, 0xcc, 0xdd, 0x8c, 0x7b, 0x13, 0xaf,
	0x9d, 0x8c, 0x8c, 0xce, 0x90, 0x4e, 0xbd, 0xfd, 0x3d, 0xd6, 0x6f, 0xbd, 0x6a, 0x5a, 0xb5, 0x51,
	0x23, 0xfb, 0x85, 0x3e, 0xf8, 0x1d, 0x00, 0x00, 0xff, 0xff, 0x16, 0xc0, 0x87, 0xdb, 0xcb, 0x05,
	0x00, 0x00,
}

// Reference imports to suppress errors if they are not otherwise used.
var _ context.Context
var _ grpc.ClientConnInterface

// This is a compile-time assertion to ensure that this generated file
// is compatible with the grpc package it is being compiled against.
const _ = grpc.SupportPackageIsVersion6

// ResponseBodyServiceClient is the client API for ResponseBodyService service.
//
// For semantics around ctx use and closing/ending streaming RPCs, please refer to https://godoc.org/google.golang.org/grpc#ClientConn.NewStream.
type ResponseBodyServiceClient interface {
	GetResponseBody(ctx context.Context, in *ResponseBodyIn, opts ...grpc.CallOption) (*ResponseBodyOut, error)
	ListResponseBodies(ctx context.Context, in *ResponseBodyIn, opts ...grpc.CallOption) (*RepeatedResponseBodyOut, error)
	ListResponseStrings(ctx context.Context, in *ResponseBodyIn, opts ...grpc.CallOption) (*RepeatedResponseStrings, error)
	GetResponseBodyStream(ctx context.Context, in *ResponseBodyIn, opts ...grpc.CallOption) (ResponseBodyService_GetResponseBodyStreamClient, error)
}

type responseBodyServiceClient struct {
	cc grpc.ClientConnInterface
}

func NewResponseBodyServiceClient(cc grpc.ClientConnInterface) ResponseBodyServiceClient {
	return &responseBodyServiceClient{cc}
}

func (c *responseBodyServiceClient) GetResponseBody(ctx context.Context, in *ResponseBodyIn, opts ...grpc.CallOption) (*ResponseBodyOut, error) {
	out := new(ResponseBodyOut)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.ResponseBodyService/GetResponseBody", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *responseBodyServiceClient) ListResponseBodies(ctx context.Context, in *ResponseBodyIn, opts ...grpc.CallOption) (*RepeatedResponseBodyOut, error) {
	out := new(RepeatedResponseBodyOut)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.ResponseBodyService/ListResponseBodies", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *responseBodyServiceClient) ListResponseStrings(ctx context.Context, in *ResponseBodyIn, opts ...grpc.CallOption) (*RepeatedResponseStrings, error) {
	out := new(RepeatedResponseStrings)
	err := c.cc.Invoke(ctx, "/grpc.gateway.examples.internal.examplepb.ResponseBodyService/ListResponseStrings", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *responseBodyServiceClient) GetResponseBodyStream(ctx context.Context, in *ResponseBodyIn, opts ...grpc.CallOption) (ResponseBodyService_GetResponseBodyStreamClient, error) {
	stream, err := c.cc.NewStream(ctx, &_ResponseBodyService_serviceDesc.Streams[0], "/grpc.gateway.examples.internal.examplepb.ResponseBodyService/GetResponseBodyStream", opts...)
	if err != nil {
		return nil, err
	}
	x := &responseBodyServiceGetResponseBodyStreamClient{stream}
	if err := x.ClientStream.SendMsg(in); err != nil {
		return nil, err
	}
	if err := x.ClientStream.CloseSend(); err != nil {
		return nil, err
	}
	return x, nil
}

type ResponseBodyService_GetResponseBodyStreamClient interface {
	Recv() (*ResponseBodyOut, error)
	grpc.ClientStream
}

type responseBodyServiceGetResponseBodyStreamClient struct {
	grpc.ClientStream
}

func (x *responseBodyServiceGetResponseBodyStreamClient) Recv() (*ResponseBodyOut, error) {
	m := new(ResponseBodyOut)
	if err := x.ClientStream.RecvMsg(m); err != nil {
		return nil, err
	}
	return m, nil
}

// ResponseBodyServiceServer is the server API for ResponseBodyService service.
type ResponseBodyServiceServer interface {
	GetResponseBody(context.Context, *ResponseBodyIn) (*ResponseBodyOut, error)
	ListResponseBodies(context.Context, *ResponseBodyIn) (*RepeatedResponseBodyOut, error)
	ListResponseStrings(context.Context, *ResponseBodyIn) (*RepeatedResponseStrings, error)
	GetResponseBodyStream(*ResponseBodyIn, ResponseBodyService_GetResponseBodyStreamServer) error
}

// UnimplementedResponseBodyServiceServer can be embedded to have forward compatible implementations.
type UnimplementedResponseBodyServiceServer struct {
}

func (*UnimplementedResponseBodyServiceServer) GetResponseBody(ctx context.Context, req *ResponseBodyIn) (*ResponseBodyOut, error) {
	return nil, status.Errorf(codes.Unimplemented, "method GetResponseBody not implemented")
}
func (*UnimplementedResponseBodyServiceServer) ListResponseBodies(ctx context.Context, req *ResponseBodyIn) (*RepeatedResponseBodyOut, error) {
	return nil, status.Errorf(codes.Unimplemented, "method ListResponseBodies not implemented")
}
func (*UnimplementedResponseBodyServiceServer) ListResponseStrings(ctx context.Context, req *ResponseBodyIn) (*RepeatedResponseStrings, error) {
	return nil, status.Errorf(codes.Unimplemented, "method ListResponseStrings not implemented")
}
func (*UnimplementedResponseBodyServiceServer) GetResponseBodyStream(req *ResponseBodyIn, srv ResponseBodyService_GetResponseBodyStreamServer) error {
	return status.Errorf(codes.Unimplemented, "method GetResponseBodyStream not implemented")
}

func RegisterResponseBodyServiceServer(s *grpc.Server, srv ResponseBodyServiceServer) {
	s.RegisterService(&_ResponseBodyService_serviceDesc, srv)
}

func _ResponseBodyService_GetResponseBody_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(ResponseBodyIn)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ResponseBodyServiceServer).GetResponseBody(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.ResponseBodyService/GetResponseBody",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ResponseBodyServiceServer).GetResponseBody(ctx, req.(*ResponseBodyIn))
	}
	return interceptor(ctx, in, info, handler)
}

func _ResponseBodyService_ListResponseBodies_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(ResponseBodyIn)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ResponseBodyServiceServer).ListResponseBodies(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.ResponseBodyService/ListResponseBodies",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ResponseBodyServiceServer).ListResponseBodies(ctx, req.(*ResponseBodyIn))
	}
	return interceptor(ctx, in, info, handler)
}

func _ResponseBodyService_ListResponseStrings_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(ResponseBodyIn)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ResponseBodyServiceServer).ListResponseStrings(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/grpc.gateway.examples.internal.examplepb.ResponseBodyService/ListResponseStrings",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ResponseBodyServiceServer).ListResponseStrings(ctx, req.(*ResponseBodyIn))
	}
	return interceptor(ctx, in, info, handler)
}

func _ResponseBodyService_GetResponseBodyStream_Handler(srv interface{}, stream grpc.ServerStream) error {
	m := new(ResponseBodyIn)
	if err := stream.RecvMsg(m); err != nil {
		return err
	}
	return srv.(ResponseBodyServiceServer).GetResponseBodyStream(m, &responseBodyServiceGetResponseBodyStreamServer{stream})
}

type ResponseBodyService_GetResponseBodyStreamServer interface {
	Send(*ResponseBodyOut) error
	grpc.ServerStream
}

type responseBodyServiceGetResponseBodyStreamServer struct {
	grpc.ServerStream
}

func (x *responseBodyServiceGetResponseBodyStreamServer) Send(m *ResponseBodyOut) error {
	return x.ServerStream.SendMsg(m)
}

var _ResponseBodyService_serviceDesc = grpc.ServiceDesc{
	ServiceName: "grpc.gateway.examples.internal.examplepb.ResponseBodyService",
	HandlerType: (*ResponseBodyServiceServer)(nil),
	Methods: []grpc.MethodDesc{
		{
			MethodName: "GetResponseBody",
			Handler:    _ResponseBodyService_GetResponseBody_Handler,
		},
		{
			MethodName: "ListResponseBodies",
			Handler:    _ResponseBodyService_ListResponseBodies_Handler,
		},
		{
			MethodName: "ListResponseStrings",
			Handler:    _ResponseBodyService_ListResponseStrings_Handler,
		},
	},
	Streams: []grpc.StreamDesc{
		{
			StreamName:    "GetResponseBodyStream",
			Handler:       _ResponseBodyService_GetResponseBodyStream_Handler,
			ServerStreams: true,
		},
	},
	Metadata: "examples/internal/proto/examplepb/response_body_service.proto",
}
